import {
  V_SIPSD_LAYER_TYPE
} from './v-sipsd-layui-layer.classes';
// import 'layui-src/dist/css/modules/layer/default/layer.css';

const VSipsdLayuiLayerService = (function () {
  function VSipsdLayuiLayerService() {
    this.loadLayerJs();
  }
  /**
   * 获取layer第三方库
   *
   *
   * \@returnses {*}
   * \@memberof VSipsdLayuiLayerService
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.getLayer = function () {
    return this.layer;
  };
  /**
   * 隐藏信息
   *
   *
   * \@params {*} layerIndx 弹出框layer的对象索引 / {number} [duration] 延迟关闭的时间，以ms(毫秒)为单位
   * \@returnses {Promise<boolean>} 关闭时返回结果
   * \@memberof VSipsdLayuiLayerService
   * @param {?} layerIndx
   * @param {?=} duration 
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.hideInfo = function (layerIndx, duration) {
    return new Promise((resolve) => {
      if (duration) {
        setTimeout(() => {
          this.layer.close(layerIndx);
          resolve(true);
        }, duration);
      } else {
        this.layer.close(layerIndx);
        resolve(true);
      }
    });
  };
  /** 加载layui-layer弹出框js
   *
   *
   * @memberof VSipsdLayuiLayerService
   */
  VSipsdLayuiLayerService.prototype.loadLayerJs = function () {
    // 如果layer还未加载过，那么就加载；
    // 加载过的话，window上会有一个全局变量layer
    if (!window['layer']) {
      require('layui-src/dist/lay/modules/layer.js');
    }
    this.layer = window['layer'];
  };
  /**
   * 显示内容提示框,没有左侧的图标
   *
   *
   * \@params {string} content 要显示的内容 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showContentInfo = function (content, tipsMore) {
    const options = {
      btn: ['确定'],
      btn1: (index) => {
        this.hideInfo(index);
      },
      content: `
      <div style="padding: 20px;text-align: left;">
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: '信息',
      type: 1,
      zIndex: 99999999
    };

    return this.showInfo(options, V_SIPSD_LAYER_TYPE.INFO);
  };
  /**
   * 简单显示错误信息
   *
   *
   * \@params {string} content 要显示的内容 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showErrorInfo = function (content, tipsMore) {
    const options = {
      btn: ['确定'],
      btn1: (index) => {
        this.hideInfo(index);
      },
      content: `
      <div style="padding: 20px 20px 20px 55px;text-align: left;">
        <i class="layui-layer-ico layui-layer-ico${V_SIPSD_LAYER_TYPE.ERROR}" 
          style="display:inline-block;width:30px;height:30px;margin-top:-4px;margin-left:-32px;vertical-align:top;"></i>
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: '注意',
      type: 1,
      zIndex: 99999999
    };

    return this.showInfo(options);
  };
  /**
   * 简单显示帮助信息
   *
   * \@params {string} content 要显示的内容 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showHelpInfo = function (content, tipsMore) {
    const options = {
      btn: ['确定'],
      btn1: (index) => {
        this.hideInfo(index);
      },
      content: `
      <div style="padding: 20px 20px 20px 55px;text-align: left;">
        <i class="layui-layer-ico layui-layer-ico${V_SIPSD_LAYER_TYPE.HELP}" 
          style="display:inline-block;width:30px;height:30px;margin-top:-4px;margin-left:-32px;vertical-align:top;"></i>
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: '注意',
      type: 1,
      zIndex: 99999999
    };

    return this.showInfo(options);
  };
  /**
   * 显示信息
   *
   *
   * \@params {*} options 配置信息 / {V_SIPSD_LAYER_TYPE} [type] 信息类型，
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} options
   * @param {?=} type
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showInfo = function (options, type) {
    if (type !== undefined && type !== null) {
      options['icon'] = type;
    }

    if (options['title'] === undefined || options['title'] === null) {
      options['title'] = '信息';
    }

    if (options['resize'] === undefined ||
      options['resize'] === null) {
      options['resize'] = false;
    }

    return this.layer.open(options);
  };
  /**
   * 简单显示常规信息
   *
   * \@params {string} content 要显示的内容 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showNormalInfo = function (content, tipsMore) {
    const options = {
      btn: ['确定'],
      btn1: (index) => {
        this.hideInfo(index);
      },
      content: `
      <div style="padding: 20px 20px 20px 55px;text-align: left;">
        <i class="layui-layer-ico layui-layer-ico${V_SIPSD_LAYER_TYPE.INFO}" 
          style="display:inline-block;width:30px;height:30px;margin-top:-4px;margin-left:-32px;vertical-align:top;"></i>
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: '信息',
      type: 1,
      zIndex: 99999999
    };

    return this.showInfo(options);
  };
  /**
   * 简单显示警告信息
   *
   * \@params {string} content 要显示的内容 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showWarnInfo = function (content, tipsMore) {
    const options = {
      btn: ['确定'],
      btn1: (index) => {
        this.hideInfo(index);
      },
      content: `
      <div style="padding: 20px 20px 20px 55px;text-align: left;">
        <i class="layui-layer-ico layui-layer-ico${V_SIPSD_LAYER_TYPE.WARN}" 
          style="display:inline-block;width:30px;height:30px;margin-top:-4px;margin-left:-32px;vertical-align:top;"></i>
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: '注意',
      type: 1,
      zIndex: 99999999
    };

    return this.showInfo(options, V_SIPSD_LAYER_TYPE.WARN);
  };
  /**
   * 显示等待状态的信息
   *
   * \@params {string} content 要显示的内容 / {number} [duration] 持续时间 / {boolean} [tipsMore] 是否多个提示框并存
   * \@returnses {*} 弹出框layer的对象索引
   * \@memberof VSipsdLayuiLayerService
   * @param {?} content
   * @param {?=} duration
   * @param {?=} tipsMore
   * @return {?}
   */
  VSipsdLayuiLayerService.prototype.showWaitingInfo = function (content, duration, tipsMore) {
    const options = {
      closeBtn: 0,
      content: `
      <div style="padding: 20px 20px 20px 55px;text-align: left;">
        <i class="layui-layer-ico layui-layer-ico${V_SIPSD_LAYER_TYPE.WAITING}" 
          style="display:inline-block;width:32px;height:32px;margin-top:-7px;margin-left:-32px;vertical-align:top;"></i>
        ${content}
      </div>`,
      shade: 0.1,
      tipsMore: !!tipsMore,
      title: false,
      type: 1,
      zIndex: 99999999
    };
    if (duration) {
      Reflect.set(options, 'time', duration);
    }
    return this.showInfo(options);
  };
  return VSipsdLayuiLayerService;
}());

export {
  VSipsdLayuiLayerService
};
