import {
  DataRequestService
} from './../../v-sipsd-tool-services';
import {
  VSipsdLayuiLayerService
} from './../../v-sipsd-layui-layer';
import {
  isLoaded,
  loadCss,
  loadScript,
  loadModules,
  // ILoadScriptOptions
} from 'esri-loader';
import {
  _
} from './../../3rd.party';
import {
  HIGHLIGHT_AREA_LAYER_INDEX,
  ICON_MARKER_LAYER_INDEX,
  ICON_MARKER_LAYER_ID,
  HIGHLIGHT_AREA_LAYER_ID,
  V_SIPSD_MAP_ARCGIS_DRAW_TYPE,
  V_SIPSD_MAP_TYPE,
} from './../ng-sipsd-map.class';
import {
  icon_dibiao_null,
  icon_dibiao_arrs
} from './../icon_dibiao_base64';
import Wkt from '../../../sipsd-map-3rd-party/wicket.arcgis.js';

var IS_LOAD_EVENT_TARGET_PATCH = false;

var VSipsdMapArcgisService = (function () {

  function VSipsdMapArcgisService() {
    this.dataRequest = new DataRequestService();
    this.layerService = new VSipsdLayuiLayerService();
  }

  /** 在地图中增加几何图形
   *
   *
   * @params {*} map
   * @params {*} geometry
   * @params {string} symbolName
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.addGeometryToMap = function (map, geometry, symbolName) {
    const modules = ['esri/graphic', 'esri/symbols/Symbol', 'esri/InfoTemplate'];
    return this.loadModules(modules).then(([Graphic, Symbol, InfoTemplate]) => {
      return this.getSymbol(geometry.type, symbolName).then((symbol) => {
        const graphic = new Graphic(geometry, symbol);
        map.graphics.add(graphic);
        return map;
      });
    });
  };

  /** 将几何图形geometry加到指定图层中
   *
   *
   * @params {*} map map对象
   * @params {string} layerId 图层id
   * @params {*} geometry 几何图形
   * @params {string} symbolName
   * @params {number} [symbolIndex]
   * @returnses
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.addGeoToGraphicLyr = function (map, layerId, geometry, popUpIndex, symbolName, symbolIndex) {
    const modules = ['esri/graphic', 'esri/layers/GraphicsLayer', ];
    return this.loadModules(modules).then(([Graphic, GraphicsLayer]) => {
      return this.getSymbol(geometry.type, symbolName, symbolIndex).then(symbol => {
        let graphicLyr = map.getLayer(layerId);
        if (!graphicLyr) {
          graphicLyr = new GraphicsLayer({
            id: layerId
          });
          map.addLayer(graphicLyr);
          // 点击时触发事件。这个一般是显示查询结果的情况。
          // 如果是i查询，不会有点击然后显示对应的气泡框的情况
          // graphicLyr.on('click', (event) => {
          //     const geometryTemp = event.graphic.geometry;
          //     if (event.graphic.attributes !== undefined) {
          //         const popUpTitleTemp = event.graphic.attributes['popUpTitle'];

          //         this.highlightGeometry(map, geometryTemp, popUpTitleTemp).then(highLightResult => {
          //             this.showMapPopUp(map, geometryTemp, popUpTitleTemp, false);
          //         });
          //     }
          // });
        }
        if ('symbol-layer' === layerId && geometry.type === 'polyline') {
          geometry = geometry.getExtent().getCenter();
        }
        const graphic = new Graphic(geometry, symbol);
        if (popUpIndex !== undefined && popUpIndex !== null) {
          graphic.setAttributes({
            popUpIndex: popUpIndex,
          });
        }

        graphicLyr.add(graphic);
        return true;
      });
    });
  };

  /** 自动加载 EventTargetPatch 内容
   *
   *
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.autoLoadEventTargetPatch = function () {
    if (!this.getIsLoadEventTargetPatch()) {
      this.loadEventTargetPatch();
    }
  };

  /** 让地图对象以一定的比例尺根据某个点居中显示
   *
   *
   * @params {*} map map对象
   * @params {number} pointX 点的x坐标
   * @params {number} pointY 点的y坐标
   * @params {number} zoomLevel 比例尺级数
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.centerAndZoom = function (map, pointX, pointY, zoomLevel) {
    const modules = ['esri/geometry/Point'];
    return this.loadModules(modules).then(([Point]) => {
      const point = new Point(pointX, pointY, map.spatialReference);
      map.centerAndZoom(point, zoomLevel);
    });
  };

  /** 让地图对象根据点信息居中显示
   *
   *
   * @params {*} map map对象
   * @params {number} pointX 点的x坐标信息
   * @params {number} pointY 点的y坐标信息
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.centerAt = function (map, pointX, pointY) {
    const modules = ['esri/geometry/Point'];
    return this.loadModules(modules).then(([Point]) => {
      const point = new Point(pointX, pointY, map.spatialReference);
      map.centerAt(point);
      return map;
    });
  };

  /** 更改权属信息
   *
   *
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.changeCopyRight = function () {
    let nodeList;
    const intervalId = setInterval(() => {
      nodeList = jQuery('.esriAttributionLastItem');
      if (nodeList.length > 0) {
        for (const item of nodeList) {
          // jQuery(item).remove();
          jQuery(item).text('园区测绘, REST API by Esri');
        }

        setTimeout(() => {
          clearInterval(intervalId);
        }, 1000);
      }
    });
  };

  /** 更改基础底图（底图和影像不共存）
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapEsriWMSLayerInfo | NgSipsdMapEsriFeatureLayerInfo | NgSipsdMapEsriTiledLayerInfo | NgSipsdMapEsriDynamicLayerInfo | NgSipsdMapVectorTiledLayerInfo | NgSipsdMapVectorGeoJsonTiledLayerInfo | NgSipsdMapTiandiTiledLayerInfo | NgSipsdMapGraphicLayerInfo | NgSipsdMapGeoJsonLayerInfo} layerInfo 图层信息
   * @params {boolean} deleteMap 是否删除map对象
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.changeMapBaseLayer = function (map, layerInfo, deleteMap, mapType, domNodeId, mapInfo, options) {
    if (deleteMap) {
      map.destroy();
      // 判断是创建单地图还是多地图
      if ('single' === mapType) {
        return this.createMap(domNodeId, options).then(callback => {
          callback['map'].on('load', () => {
            this.setInitExtent(callback['map'], mapInfo);
          });
          return callback['map'];
        });
      } else if ('double' === mapType) {
        return this.createDoubleMap(domNodeId, options).then(callback => {
          callback['map'].on('load', () => {
            this.setInitExtent(callback['map'], mapInfo);
          });
          return callback['map'];
        });
      }
    } else {
      // 不用再清除原来的底图，因为外部自动会执行清除地图的操作
      // const baseLayerId = map.layerIds[0];
      // this.removeMapLayer(map, baseLayerId);
      return this.loadMapLayer(map, layerInfo, 0).then((callback) => {
        return {
          map: callback
        };
      });
    }
  };
  /** 检查返回的图层信息是否有效
   *
   *
   * @params {string} layerUrl
   * @returnses {Promise<boolean>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.checkLayerInfoValid = function (layerUrl) {
    return this.dataRequest.getJsonDataMethodByPromise(layerUrl).then(validResult => {
      let {
        status,
        responseData
      } = validResult;
      if (200 === status) {
        // 当是动态图层时，因为请求的结果是显示动态图层具体信息的html页面，所以无法转成json，
        // 因此这边得到的值是undefined，这种情况下，直接默认为合法的信息
        if (responseData) {
          // 网关的异常状态码
          // if (40101 === responseData['status'] ||
          //   40102 === responseData['status'] ||
          //   40131 === responseData['status'] ||
          //   40331 === responseData['status']) {
          //   const content = responseData['message'];
          //   this.layerService.showErrorInfo(content);
          //   return false;
          // }
          if (responseData['status']) {
            const content = JSON.stringify(responseData);
            this.layerService.showErrorInfo(content);
            return false;
          }
          // 原来平台的地图服务报错的情况的处理，格式：
          // {Status: "500.3.1", Des: "令牌验证错误!", Msg: "令牌[token]验证错误，请检查令牌是否正确!"}
          if (responseData['Status']) {
            const content = responseData['Msg'];
            this.layerService.showErrorInfo(content);
            return false;
          }
          return true;
        }
        return true;
      } else {
        return false;
      }
    });
  };

  /** 检查地图服务地址是否是合法有效的
   *
   *
   * @params {string} layerUrl 地图服务地址
   * @returnses {boolean} 合法有效的地址返回true，不合法无效的返回false
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.checkLayerUrlValid = function (layerUrl) {
    if (layerUrl === undefined || layerUrl === null || '' === layerUrl.trim()) {
      this.layerService.showWarnInfo(`“${layerUrl}”为无效的地图服务地址，无法加载该图层！`);
      return false;
    }
    return true;
  };

  /** 清除地图上的图形信息
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Object}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.clearGraphics = function (map) {
    if (map) {
      map.setMapCursor('default');
      map.infoWindow.hide();
      map.graphics.clear();

      const graphicsLyrIds = map.graphicsLayerIds;
      _.forEach(graphicsLyrIds, (graphicsLyrId) => {
        const graphicsLyr = map.getLayer(graphicsLyrId);
        graphicsLyr.clear();
      });
    }
    return {
      map: map,
    };
  };

  /** 清除高亮的图层
   *
   *
   * @params {*} map
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.clearHightlightGraphics = function (map) {
    const highlightGLyr = map.getLayer('highlightGLyr');
    if (highlightGLyr !== undefined) {
      highlightGLyr.clear();
    }
    return map;
  };

  /** 在地图上，根据图形id清除对应的图形信息
   *
   *
   * @params {*} map map对象
   * @params {*} graphicsLyrId
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.clearLayerGraphics = function (map, graphicsLyrId) {
    map.infoWindow.hide();
    const graphicsLyr = map.getLayer(graphicsLyrId);
    if (graphicsLyr) {
      graphicsLyr.clear();
    }
    return map;
  };
  /** 清除气泡
   *
   *
   * @params {*} map
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.clearPopUps = function (map) {
    // 清除定位图标所在的图层
    const symbolLayer = map.getLayer(ICON_MARKER_LAYER_ID);
    if (symbolLayer) {
      map.removeLayer(symbolLayer);
    }
    // 清除高亮区域图层
    const highlightGLyr = map.getLayer(HIGHLIGHT_AREA_LAYER_ID);
    if (highlightGLyr) {
      map.removeLayer(highlightGLyr);
    }
    // 清除气泡框
    map.infoWindow.hide();
  };
  /** 将geometry类型的数据转成wkt类型的数据
   *
   *
   * @params {*} geometry
   * @returnses {string}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.convertGeometry2Wkt = function (geometry) {
    const wkt = new Wkt.Wkt();
    wkt.fromObject(geometry);
    return wkt.write();
  };
  /** 将wkt类型的数据转换成geometry类型的数据
   *
   *
   * @params {*} map
   * @params {string} wktStr
   * @returnses {Object}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.convertWkt2Geometry = function (map, wktStr) {
    const wkt = new Wkt.Wkt();
    wkt.read(wktStr);
    const config = {
      spatialReference: {
        wkt: map.spatialReference.wkt
      },
      editable: false
    };
    return wkt.toObject(config);
  };
  /** 创建双地图
   *
   *
   * @params {string} domNodeId
   * @params {ArcgisMapMapInfo} mapInfo
   * @params {ILoadScriptOptions} [loadScriptOptions]
   * @returnses
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createDoubleMap = function (domNodeId, loadScriptOptions) {
    const modules = ['esri/map'];
    return this.loadModules(modules, loadScriptOptions).then(([Map]) => {
      const options = {
        logo: false,
        slider: false
      };

      const map = new Map(domNodeId, options);
      return {
        map: map
      };
    });
  };
  /** 创建高亮区域项
   *
   *
   * @params {*} geometry
   * @params {*} geometryStyle
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createHighlightAreaItem = function (geometry, geometryStyle, constructorClass) {
    let style;
    const geometryType = geometry.type,
      defaultStyle = {
        fill: {
          color: '#3199ff',
          opacity: 0.5
        },
        border: {
          color: '#3199ff',
          width: 2
        }
      },
      Graphic = constructorClass['Graphic'],
      Color = constructorClass['Color'],
      SimpleMarkerSymbol = constructorClass['SimpleMarkerSymbol'],
      SimpleLineSymbol = constructorClass['SimpleLineSymbol'],
      SimpleFillSymbol = constructorClass['SimpleFillSymbol'];
    if (!geometryStyle) {
      geometryStyle = defaultStyle;
    }
    if (geometryStyle.customStyle) {
      style = geometryStyle.customStyle;
    } else {
      if (!geometryStyle.fill) {
        geometryStyle.fill = defaultStyle.fill;
      } else {
        if (!geometryStyle.fill.color) {
          geometryStyle.fill.color = defaultStyle.fill.color;
        }
        if (geometryStyle.fill.opacity === undefined || geometryStyle.fill.opacity === null) {
          geometryStyle.fill.opacity = defaultStyle.fill.opacity;
        }
      }
      if (!geometryStyle.border) {
        geometryStyle.border = defaultStyle.border;
      } else {
        if (!geometryStyle.border.color) {
          geometryStyle.border.color = defaultStyle.border.color;
        }
        if (geometryStyle.border.width === undefined || geometryStyle.border.width === null) {
          geometryStyle.border.width = defaultStyle.border.width;
        }
      }
      let fillColor = (new Color(geometryStyle.fill.color)).toRgba(),
        borderColor = new Color(geometryStyle.border.color);
      fillColor[3] = geometryStyle.fill.opacity;
      fillColor = new Color(fillColor);
      if ('point' === geometryType) {
        style = new SimpleMarkerSymbol({
          color: fillColor,
          size: geometryStyle.border.width,
          outline: {
            color: borderColor,
            width: geometryStyle.border.width
          },
        });
      } else if ('polyline' === geometryType) {
        style = new SimpleLineSymbol(
          SimpleLineSymbol.STYLE_SOLID,
          borderColor,
          geometryStyle.border.width
        );
      } else if ('polygon' === geometryType) {
        style = new SimpleFillSymbol({
          type: 'esriSFS',
          style: 'esriSFSSolid',
          color: fillColor,
          outline: {
            type: 'esriSLS',
            style: 'esriSLSSolid',
            color: borderColor,
            width: geometryStyle.border.width
          }
        });
      }

    }

    return new Graphic(geometry, style);
  };
  /** 创建高亮区域图层
   *
   *
   * @params {*} map
   * @params {Object[]} highlightAreaList
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createHighlightAreaLayer = function (map, highlightAreaList, constructorClass) {
    const layerId = HIGHLIGHT_AREA_LAYER_ID,
      GraphicsLayerClass = constructorClass['GraphicsLayer'];
    let highlightAreaLayer = map.getLayer(layerId);
    if (highlightAreaLayer) {
      this.removeMapLayer(map, layerId);
    }
    highlightAreaLayer = new GraphicsLayerClass({
      id: layerId
    });
    map.addLayer(highlightAreaLayer, HIGHLIGHT_AREA_LAYER_INDEX);

    for (let i = 0; i < highlightAreaList.length; i++) {
      const item = highlightAreaList[i],
        geometry = this.convertWkt2Geometry(map, item['geometry']);
      const areaItem = this.createHighlightAreaItem(geometry, item['geometryStyle'], constructorClass);
      highlightAreaLayer.add(areaItem);
    }

    return {
      map: map,
      layer: highlightAreaLayer,
    };
  };
  /** 创建带有geometry的索引图标标识
   *
   *
   * @params {number} index
   * @params {*} geometry
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createIndexIconMarkerItemWidthGeometry = function (index, geometry, constructorClass) {
    const iconPath = icon_dibiao_arrs[index],
      PictureMarkerSymbol = constructorClass['PictureMarkerSymbol'],
      Graphic = constructorClass['Graphic'];
    if (iconPath) {
      const symbol = new PictureMarkerSymbol(iconPath, 30, 30);
      symbol.setOffset(0, 12);
      if (geometry.type === 'polyline') {
        geometry = geometry.getExtent().getCenter();
      }
      return new Graphic(geometry, symbol);;
    } else {
      return this.createNullIndexIconMarkerItemWidthGeometry(geometry, constructorClass);
    }
  };
  /** 创建索引图标标识的图层
   *
   *
   * @params {*} map 地图实例
   * @params {string} layerId 图层id
   * @params {Object[]} indexIconList 索引图标数组
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createIndexIconMarkersLayer = function (map, layerId, indexIconList, constructorClass) {
    let indexIconMarkersLayer = map.getLayer(layerId);
    if (indexIconMarkersLayer) {
      this.removeMapLayer(map, layerId);
    }
    const GraphicsLayer = constructorClass['GraphicsLayer'];
    indexIconMarkersLayer = new GraphicsLayer({
      id: layerId
    });
    map.addLayer(indexIconMarkersLayer, ICON_MARKER_LAYER_INDEX);

    for (let i = 0; i < indexIconList.length; i++) {
      const item = indexIconList[i],
        geometry = this.convertWkt2Geometry(map, item['geometry']);
      const markerItem = this.createIndexIconMarkerItemWidthGeometry(i, geometry, constructorClass);
      markerItem.setAttributes({
        index: i,
      });
      indexIconMarkersLayer.add(markerItem);
    }

    return {
      map: map,
      layer: indexIconMarkersLayer,
    }
  };
  /** 创建map对象
   *
   *
   * @params {string} domNodeId dom节点的id
   * @params {ILoadScriptOptions} [loadScriptOptions] 配置信息
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createMap = function (domNodeId, loadScriptOptions) {
    const modules = ['esri/map', 'esri/dijit/Scalebar'];
    return this.loadModules(modules, loadScriptOptions).then(([EsriMap, EsriDijitScalebar]) => {
      return new Promise(resolve => {
        const options = {
          logo: false,
          slider: false
        };
        setTimeout(() => {
          const map = new EsriMap(domNodeId, options);
          const scalebar = new EsriDijitScalebar({
            map: map,
            scalebarStyle: 'line',
            scalebarUnit: 'dual'
          });

          resolve({
            map: map,
            esriDijitScalebar: scalebar
          });
        })
      });
    });
  };
  /** 创建带有geometry的无索引图标标识
   *
   *
   * @params {*} geometry
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createNullIndexIconMarkerItemWidthGeometry = function (geometry, constructorClass) {
    const iconPath = icon_dibiao_null,
      Graphic = constructorClass['Graphic'],
      PictureMarkerSymbol = constructorClass['PictureMarkerSymbol'];
    const symbol = new PictureMarkerSymbol(iconPath, 20, 21);
    symbol.setOffset(0, 8);
    if (geometry.type === 'polyline') {
      geometry = geometry.getExtent().getCenter();
    }
    return new Graphic(geometry, symbol);;
  };
  /** 创建无索引图标标识的图层
   *
   *
   * @params {*} map
   * @params {string} layerId
   * @params {Object[]} nullIndexIconList
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.createNullIndexIconMarkersLayer = function (map, layerId, nullIndexIconList, constructorClass) {
    let nullIndexIconMarkersLayer = map.getLayer(layerId);
    if (nullIndexIconMarkersLayer) {
      this.removeMapLayer(map, layerId);
    }
    const GraphicsLayer = constructorClass['GraphicsLayer'];
    nullIndexIconMarkersLayer = new GraphicsLayer({
      id: layerId
    });
    map.addLayer(nullIndexIconMarkersLayer, ICON_MARKER_LAYER_INDEX);

    for (let i = 0; i < nullIndexIconList.length; i++) {
      const item = nullIndexIconList[i],
        geometry = this.convertWkt2Geometry(map, item['geometry']);
      const markerItem = this.createNullIndexIconMarkerItemWidthGeometry(geometry, constructorClass);
      markerItem.setAttributes({
        index: i,
      });
      nullIndexIconMarkersLayer.add(markerItem);
    }

    return {
      map: map,
      layer: nullIndexIconMarkersLayer,
    }
  };
  /** 使原来的画图工具对象失效
   *
   *
   * @params {*} drawTool
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.deactivateDrawTool = function (drawTool) {
    if (drawTool) {
      drawTool.deactivate();
    }
    return drawTool;
  };
  /** 在地图中绘制图形
   *
   *
   * @params {*} map map对象
   * @params {V_SIPSD_MAP_ARCGIS_DRAW_TYPE} type 图形类型
   * @params {*} drawTool 画图工具对象
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.drawGraphics = function (map, type) {
    const modules = ['esri/toolbars/draw'];
    return this.loadModules(modules).then(([Draw]) => {
      // drawTool = this.deactivateDrawTool(drawTool);
      const drawTool = new Draw(map);
      map.setMapCursor('crosshair');
      map.disableMapNavigation();
      drawTool.activate(Draw[type]);

      return new Promise((resolve, reject) => {
        drawTool.onDrawComplete = (event) => {
          drawTool.deactivate();
          map.setMapCursor('default');
          map.enableMapNavigation();
          this.addGeometryToMap(map, event.geometry, 'default_symbol').then((callbackMap) => {
            map = callbackMap;
            resolve({
              map: map,
              // drawTool: drawTool,
              geometry: event.geometry
            });
          });
        };
      });
    });
  };
  /** 获取在地图上鼠标单击的点的wkt类型的值
   *
   *
   * @params {*} map map对象
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getClickPoint = function (map) {
    return new Promise((resolve, reject) => {
      map.setMapCursor('help');
      map.onClick = (e) => {
        map.setMapCursor('default');
        const pointWkt = this.convertGeometry2Wkt(e.mapPoint);

        // 解决map.popUp.component不执行ngOnInit，大致用途就是让dom重新渲染下
        // map.resize();

        resolve({
          map: map,
          value: pointWkt
        });
      };
    });
  };
  /** 获取默认的符号信息
   *
   *
   * @params {string} type 类型
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getDefaultSymbol = function (type) {
    const modules = [
      'esri/symbols/Symbol',
      'esri/symbols/SimpleMarkerSymbol',
      'esri/symbols/SimpleLineSymbol',
      'esri/symbols/SimpleFillSymbol',
    ];
    return this.loadModules(modules).then(
      ([Symbol, SimpleMarkerSymbol, SimpleLineSymbol, SimpleFillSymbol]) => {
        let symbol;
        switch (type) {
          case 'point':
          case 'multipoint':
            symbol = new SimpleMarkerSymbol();
            break;
          case 'polyline':
            symbol = new SimpleLineSymbol();
            break;
          case 'polygon':
            symbol = new SimpleFillSymbol();
            break;
        }
        return symbol;
      }
    );
  };


  /** 获取地图范围
   *
   *
   * @params {*} map 地图实例
   * @returnss {Object}
   * @memberof NgSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getExtent = function (map) {
    return new Promise(resolve => {
      let intervalIndex = setInterval(() => {
        if (map.extent) {
          clearInterval(intervalIndex);
          intervalIndex = null;
          let {
            xmin,
            ymin,
            xmax,
            ymax,
            spatialReference
          } = map.extent;
          spatialReference = spatialReference.wkt;
          resolve({
            map: map,
            value: {
              xmin,
              ymin,
              xmax,
              ymax,
              spatialReference
            }
          });
        }
      });
      // 3s后还无法获取范围值，则不再获取
      setTimeout(() => {
        if (intervalIndex) {
          clearInterval(intervalIndex);
          resolve({
            map: map,
            value: null
          });
        }
      }, 3000);
    });
  };
  /** 是否已经加载 EventTargetPatch
   *
   *
   * @returnses {boolean}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getIsLoadEventTargetPatch = function () {
    return _.cloneDeep(IS_LOAD_EVENT_TARGET_PATCH);
  };
  /** 获取当前比例尺层级
   *
   *
   * @params {*} map
   * @returnses {number}
   * @memberof NgSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getLevel = function (map) {
    return map.getLevel();
  };
  /** 从图标标识图层中根据索引获取指定的图标标识
   *
   *
   * @params {*} map
   * @params {number} index
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getMarkerItemFromIconMarkersLayer = function (map, index) {
    let markerItem = null;
    const iconMarkersLayer = map.getLayer(ICON_MARKER_LAYER_ID);
    if (iconMarkersLayer) {
      const graphics = iconMarkersLayer.graphics;
      for (const item of graphics) {
        if (item.attributes && index === item.attributes.index) {
          markerItem = item;
          break;
        }
      }
    }
    return markerItem;
  };
  /** 获取符号
   *
   *
   * @params {string} geometryType
   * @params {string} symbolName
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.getSymbol = function (geometryType, symbolName, index) {
    const modules = [
      'esri/Color',
      'esri/symbols/Symbol',
      'esri/symbols/SimpleMarkerSymbol',
      'esri/symbols/PictureMarkerSymbol',
      'esri/symbols/SimpleLineSymbol',
      'esri/symbols/SimpleFillSymbol',
    ];
    return this.loadModules(modules).then(
      ([Color, Symbol, SimpleMarkerSymbol, PictureMarkerSymbol, SimpleLineSymbol, SimpleFillSymbol]) => {
        let symbol,
          iconPath;
        const symbolId = `${geometryType}_${symbolName}`;
        switch (symbolId) {
          /** default symbol */
          case 'point_default_symbol':
          case 'multipoint_default_symbol':
            {
              symbol = new SimpleMarkerSymbol();
              break;
            }
          case 'polyline_default_symbol':
            {
              symbol = new SimpleLineSymbol();
              break;
            }
          case 'polygon_default_symbol':
            {
              symbol = new SimpleFillSymbol();
              break;
            }
          case 'point_highlight_symbol':
            {
              // iconPath = `assets/libs/sipsd/sipsdMap/poi/icon_dibiao_${index}.png`;
              // symbol = new PictureMarkerSymbol(iconPath, 30, 30);
              symbol = new SimpleMarkerSymbol({
                // yoffset: -12,
                'color': [52, 52, 52, 0],
                size: 8,
                'outline': {
                  color: [255, 0, 0, 120],
                  width: 2
                },
              });
              break;
            }
          case 'polyline_highlight_symbol':
            {
              symbol = new SimpleLineSymbol(SimpleLineSymbol.STYLE_SOLID, new Color([253, 128, 68]), 3);
              break;
            }
          case 'polygon_highlight_symbol':
            {
              symbol = new SimpleFillSymbol({
                'type': 'esriSFS',
                'style': 'esriSFSSolid',
                'color': [255, 0, 0, 120],
                'outline': {
                  'type': 'esriSLS',
                  'style': 'esriSLSSolid',
                  'color': [255, 0, 0, 255],
                  'width': 2
                }
              });
              break;
            }
            /** custom symbol */
          case 'polyline_dibiao_picture_marker_symbol':
          case 'point_dibiao_picture_marker_symbol':
          case 'polygon_dibiao_picture_marker_symbol':
            {
              // iconPath = `assets/libs/sipsd/sipsdMap/poi/icon_dibiao_null.png`;
              iconPath = icon_dibiao_null;
              symbol = new PictureMarkerSymbol(iconPath, 20, 21);
              symbol.setOffset(0, 8);
              break;
            }
          case 'polyline_index_picture_marker_symbol':
          case 'point_index_picture_marker_symbol':
          case 'polygon_index_picture_marker_symbol':
            {
              // iconPath = `assets/libs/sipsd/sipsdMap/poi/icon_dibiao_${index}.png`;
              // 数组从0开始，所以这边索引要减去1
              iconPath = icon_dibiao_arrs[index - 1];
              symbol = new PictureMarkerSymbol(iconPath, 30, 30);
              symbol.setOffset(0, 12);
              break;
            }
          case 'polyline_measure_symbol':
            {
              symbol = new SimpleLineSymbol(
                SimpleLineSymbol.STYLE_SOLID,
                new Color([253, 128, 68]),
                3
              );
              break;
            }
          case 'polygon_measure_symbol':
            {
              const lineSymbol = new SimpleLineSymbol(
                SimpleLineSymbol.STYLE_SOLID,
                new Color([253, 128, 68]),
                2
              );
              symbol = new SimpleFillSymbol(
                SimpleFillSymbol.STYLE_SOLID,
                lineSymbol,
                new Color([0, 255, 123, 0.4])
              );
              break;
            }
        }
        return symbol;
      }
    );
  };
  /** 高亮显示地图气泡框
   *
   *
   * @params {*} map
   * @params {*} geometry
   * @params {number} popUpIndex
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.highlightGeometry = function (map, geometry, popUpIndex) {
    const modules = ['esri/layers/GraphicsLayer', ];
    return this.loadModules(modules).then(([GraphicsLayer]) => {
      let highlightGLyr = map.getLayer('highlightGLyr');
      if (highlightGLyr) {
        map = this.clearHightlightGraphics(map);
        // 放到最顶层
        map.reorderLayer(highlightGLyr, 0);
      } else {
        highlightGLyr = new GraphicsLayer({
          id: 'highlightGLyr'
        });
        map.addLayer(highlightGLyr);
      }

      const symbolLayer = map.getLayer('symbol-layer');
      if (symbolLayer) {
        // 将选中结果对应的定位图标置顶，
        const selectedGraphic = symbolLayer.graphics[popUpIndex];
        symbolLayer.remove(selectedGraphic);
        symbolLayer.add(selectedGraphic);
      }

      return this.addGeoToGraphicLyr(map, 'highlightGLyr', geometry, popUpIndex, 'highlight_symbol').then(theResult => {
        return theResult;
      });
    });
  };
  /** esri的相关配置是否已经加载
   *
   *
   * @returnses {*}
   * @memberof BaseFuncService
   */
  VSipsdMapArcgisService.prototype.isLoaded = function () {
    return isLoaded();
  };
  /** 加载Arcgis所需的css样式
   *
   *
   * @params {string} cssUrl 样式链接
   * @returnses {HTMLLinkElement}
   * @memberof BaseFuncService
   */
  VSipsdMapArcgisService.prototype.loadCss = function (cssUrl) {
    return loadCss(cssUrl);
  };
  /** 在map对象中加载默认图层
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapEsriWMSLayerInfo | NgSipsdMapEsriFeatureLayerInfo | NgSipsdMapEsriTiledLayerInfo | NgSipsdMapEsriDynamicLayerInfo | NgSipsdMapVectorTiledLayerInfo | NgSipsdMapVectorGeoJsonTiledLayerInfo | NgSipsdMapTiandiTiledLayerInfo | NgSipsdMapGraphicLayerInfo | NgSipsdMapGeoJsonLayerInfo} defaultlayer 默认图层信息
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadDefaultLayers = function (map, defaultlayer) {
    return this.loadMapLayer(map, defaultlayer, defaultlayer.index);
  };
  /** 在map对象中加载动态图层
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapEsriDynamicLayerInfo} layerInfo 图层信息
   * @params {number} [index] 层级
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadDynamicLayer = function (map, layerInfo, index) {
    const modules = ['esri/layers/ArcGISDynamicMapServiceLayer'];
    return this.loadModules(modules).then(([ArcGISDynamicMapServiceLayer]) => {
      let layerUrl = Reflect.get(layerInfo, 'url');
      if (!this.checkLayerUrlValid(layerUrl)) {
        return {
          map: map,
          layer: undefined
        };
      }
      const params = Reflect.get(layerInfo, 'params');
      const paramsKeys = params ? Object.keys(params) : [];
      let validLayerInfoUrl;
      if (paramsKeys.length > 0) {
        layerUrl = `${layerUrl}?${paramsKeys[0]}=${encodeURI(Reflect.get(params, paramsKeys[0]))}`;
        for (let i = 1; i < paramsKeys.length; i++) {
          layerUrl = `${layerUrl}&${paramsKeys[i]}=${encodeURI(Reflect.get(params, paramsKeys[i]))}`;
        }
        // 这边图层地址要加上'&f=json'，表示信息以json格式返回回来
        validLayerInfoUrl = `${layerUrl}&f=json`;
      } else {
        // 这边图层地址要加上'&f=json'，表示信息以json格式返回回来
        validLayerInfoUrl = `${layerUrl}?f=json`;
      }

      return new Promise((resolve) => {
        if (layerInfo.checkValid === true) {
          this.checkLayerInfoValid(validLayerInfoUrl).then(validResult => {
            resolve(validResult);
          });
        } else {
          resolve(true);
        }
      }).then(validResult => {
        let dynamicLyr;
        if (validResult) {
          dynamicLyr = new ArcGISDynamicMapServiceLayer(layerUrl, {
            id: layerInfo.id
          });
          if (layerInfo.vls) {
            dynamicLyr.setVisibleLayers(layerInfo.vls);
          }
          map.addLayer(dynamicLyr, index);
        }
        return {
          map: map,
          layer: dynamicLyr
        };
      });
    });
  };
  /** 加载EventTargetPatch，解决zone.js的版本>0.8.12时，
   * 在ie下Arcgis地图点击移动，测量距离等双击结束功能等无法
   * 正常使用的问题
   *
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadEventTargetPatch =
    function () {
      if (typeof EventTarget === 'undefined') {
        // zone.js版本>0.8.12时，arcgis地图在ie11下测量距离等需要双击的功能无法正常使用，此时可添加以下代码解决
        window.Zone.__load_patch('EventTargetPatch', (global, Zone, api) => {
          IS_LOAD_EVENT_TARGET_PATCH = true;
          const apis = [];
          if (typeof EventTarget === 'undefined') {
            const NO_EVENT_TARGET = `ApplicationCache,EventSource,FileReader,InputMethodContext,MediaController,MessagePort,Node,Performance,SVGElementInstance,SharedWorker,TextTrack,TextTrackCue,TextTrackList,WebKitNamedFlow,Window,Worker,WorkerGlobalScope,XMLHttpRequest,XMLHttpRequestEventTarget,XMLHttpRequestUpload,IDBRequest,IDBOpenDBRequest,IDBDatabase,IDBTransaction,IDBCursor,DBIndex,WebSocket,XMLHttpRequestEventTarget`
              .split(',').forEach(e => {
                const addSymbol = api.symbol('addEventListener');
                const removeSymbol = api.symbol('removeEventListener');
                const T = global[e];
                if (T && T.prototype) {
                  T.prototype.addEventListener = T.prototype[addSymbol];
                  T.prototype.removeEventListener = T.prototype[removeSymbol];

                  T.prototype[addSymbol] = null;
                  T.prototype[removeSymbol] = null;
                }
                apis.push(e);
              });
          } else {
            return;
          }
          api.patchEventTarget(global, apis, {
            useG: false
          });
        });
      }
    };
  /** 在map对象中加载特性图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriFeatureLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadFeatureLayer = function (map, layerInfo, index) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('feature layer加载未实现！');
      resolve({
        map: map
      });
    });
  };
  /** 加载返回数据是GeoJson格式的数据的图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapGeoJsonLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadGeoJsonLayer = function (map, layerInfo, index) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('geoJson layer加载未实现！');
      resolve({
        map: map
      });
    });
  };
  /** 加载图形图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapGraphicLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadGraphicLayer = function (map, layerInfo, index) {
    const modules = [
      'esri/graphic',
      'esri/layers/GraphicsLayer',
      'esri/symbols/SimpleMarkerSymbol',
      'esri/symbols/SimpleLineSymbol',
      'esri/symbols/SimpleFillSymbol',
      'esri/Color',
    ];
    return this.loadModules(modules).then(([
      Graphic,
      GraphicsLayer,
      SimpleMarkerSymbol,
      SimpleLineSymbol,
      SimpleFillSymbol,
      Color,
    ]) => {
      let graphicLyr;
      if (layerInfo.graphicList) {
        graphicLyr = map.getLayer(layerInfo.id);
        if (!graphicLyr) {
          graphicLyr = new GraphicsLayer({
            id: layerInfo.id
          });
          for (const item of layerInfo.graphicList) {
            const geometryData = this.convertWkt2Geometry(map, item.geometry);
            let style,
              geometryStyle = item.style;
            const geometryType = geometryData['type'],
              defaultStyle = {
                fill: {
                  color: '#3199ff',
                  opacity: 0.5
                },
                border: {
                  color: '#3199ff',
                  width: 2
                }
              }
            if (!geometryStyle) {
              geometryStyle = defaultStyle;
            }
            if (geometryStyle.customStyle) {
              style = geometryStyle.customStyle;
            } else {
              if (!geometryStyle.fill) {
                geometryStyle.fill = defaultStyle.fill;
              } else {
                if (!geometryStyle.fill.color) {
                  geometryStyle.fill.color = defaultStyle.fill.color;
                }
                if (geometryStyle.fill.opacity === undefined || geometryStyle.fill.opacity === null) {
                  geometryStyle.fill.opacity = defaultStyle.fill.opacity;
                }
              }
              if (!geometryStyle.border) {
                geometryStyle.border = defaultStyle.border;
              } else {
                if (!geometryStyle.border.color) {
                  geometryStyle.border.color = defaultStyle.border.color;
                }
                if (geometryStyle.border.width === undefined || geometryStyle.border.width === null) {
                  geometryStyle.border.width = defaultStyle.border.width;
                }
              }
              let fillColor = (new Color(geometryStyle.fill.color)).toRgba(),
                borderColor = new Color(geometryStyle.border.color);
              fillColor[3] = geometryStyle.fill.opacity;
              fillColor = new Color(fillColor);
              if ('point' === geometryType) {
                style = new SimpleMarkerSymbol({
                  color: fillColor,
                  size: geometryStyle.border.width,
                  outline: {
                    color: borderColor,
                    width: geometryStyle.border.width
                  },
                });
              } else if ('polyline' === geometryType) {
                style = new SimpleLineSymbol(
                  SimpleLineSymbol.STYLE_SOLID,
                  borderColor,
                  geometryStyle.border.width
                );
              } else if ('polygon' === geometryType) {
                style = new SimpleFillSymbol({
                  type: 'esriSFS',
                  style: 'esriSFSSolid',
                  color: fillColor,
                  outline: {
                    type: 'esriSLS',
                    style: 'esriSLSSolid',
                    color: borderColor,
                    width: geometryStyle.border.width
                  }
                });
              }
            }
            const graphic = new Graphic(geometryData, style);
            graphicLyr.add(graphic);
          }
          map.addLayer(graphicLyr, index);
        } else {
          this.layerService.showErrorInfo(`id为${layerInfo.id}的“${layerInfo.name}”图层已存在,无法重复加载!`);
        }
      } else {
        this.layerService.showErrorInfo('图层的graphicList图形列表数据缺失，无法加载该图层！');
      }
      return {
        map: map,
        layer: graphicLyr
      };
    });
  };
  /** 在map对象中加载图层
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapEsriWMSLayerInfo | NgSipsdMapEsriFeatureLayerInfo | NgSipsdMapEsriTiledLayerInfo | NgSipsdMapEsriDynamicLayerInfo | NgSipsdMapVectorTiledLayerInfo | NgSipsdMapVectorGeoJsonTiledLayerInfo | NgSipsdMapTiandiTiledLayerInfo | NgSipsdMapGraphicLayerInfo | NgSipsdMapGeoJsonLayerInfo} layerInfo 图层信息
   * @params {number} [index] 层级
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadMapLayer = function (map, layerInfo, index) {
    if (layerInfo && layerInfo.url) {
      const layerUrlList = layerInfo.url.split('?');
      if (layerUrlList.length > 1 && layerUrlList[1]) {
        const paramUrlList = layerUrlList[1].split('&');
        paramUrlList.forEach(paramUrl => {
          const paramList = paramUrl.split('=');
          if (!layerInfo.params) {
            layerInfo.params = {};
          }
          Reflect.set(layerInfo.params, paramList[0], paramList[1]);
        });
      }
      layerInfo.url = layerUrlList[0];
    }

    const layerInfoType = layerInfo.type.toLowerCase();
    if (V_SIPSD_MAP_TYPE.ESRI_TILEDLAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.TILE_SERVICES.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.ARCGIS_TILE.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.TILE.toLowerCase() === layerInfoType) {
      return this.loadTiledLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.VECTOR_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadVectorTiledLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.ESRI_DYNAMICLAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.ARCGIS_DYN.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.DYNAMIC.toLowerCase() === layerInfoType) {
      return this.loadDynamicLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.ESRI_FEATURELAYER.toLowerCase() === layerInfoType) {
      return this.loadFeatureLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.TIANDI_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadTianMapLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.ESRI_WMSLAYER.toLowerCase() === layerInfoType) {
      return this.loadWMSLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.GRAPHIC_LAYER.toLowerCase() === layerInfoType) {
      return this.loadGraphicLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.GEOJSON_LAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.GEOJSON.toLowerCase() === layerInfoType) {
      return this.loadGeoJsonLayer(map, layerInfo, index);
    } else if (V_SIPSD_MAP_TYPE.VECTOR_GEOJSON_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadVectorGeoJsonTiledLayer(map, layerInfo, index);
    } else {
      return new Promise((resolve, reject) => {
        this.layerService.showWarnInfo(`“${layerInfoType}”为未支持的地图类型，无法加载该图层。请开发者通过设置图层信息中的“customLoad”属性，进行自定义加载。详情请见组件使用说明！`);
        // console.warn(`“${layerInfo.type}”为未支持的地图类型，在组件中通过“ngSingleMapSelectLayersTreeChange”方法获得加载的图层列表后，请自行实现加载方法！`);
        resolve({
          map: map
        });
      });
    }
  };
  /** 懒加载地图模块.默认引用的是最新的arcgisApi(目前是4.X)。
   * 如果想引用旧版的arcgisApi(比如3.23)，可以在相关配置中设置：
   * loadScriptOptions = {
   *  url:'https://js.arcgis.com/3.23/'
   * }
   *
   * @params {string[]} modules 所需的地图模块名称
   * @params {ILoadScriptOptions} [loadScriptOptions] 相关配置
   * @returnses {Promise<any[]>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadModules = function (modules, loadScriptOptions) {
    return loadModules(modules, loadScriptOptions);
  };
  /** 加载arcgis的初始化脚本
   *
   *
   * @params {ILoadScriptOptions} [options]
   * @returnses {Promise<HTMLScriptElement>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadScript = function (options) {
    return loadScript(options);
  };
  /** 加载天地图
   *
   *
   * @params {*} map
   * @params {NgSipsdMapTiandiTiledLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadTianMapLayer = function (map, layerInfo, index) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('天地图加载未实现！');
      resolve({
        map: map
      });
    });
  };
  /** 在map对象中加载切片图层
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapEsriTiledLayerInfo} layerInfo 图层信息
   * @params {number} [index] 层级
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadTiledLayer =
    function (map, layerInfo, index) {
      const modules = ['esri/layers/ArcGISTiledMapServiceLayer'];
      return this.loadModules(modules).then(([EsriLayersArcGISTiledMapServiceLayer]) => {
        let layerUrl = Reflect.get(layerInfo, 'url');
        if (!this.checkLayerUrlValid(layerUrl)) {
          return {
            map: map,
            layer: undefined
          };
        }
        const params = Reflect.get(layerInfo, 'params');
        const paramsKeys = params ? Object.keys(params) : [];
        let validLayerInfoUrl;
        if (paramsKeys.length > 0) {
          layerUrl = `${layerUrl}?${paramsKeys[0]}=${encodeURI(Reflect.get(params, paramsKeys[0]))}`;
          for (let i = 1; i < paramsKeys.length; i++) {
            layerUrl = `${layerUrl}&${paramsKeys[i]}=${encodeURI(Reflect.get(params, paramsKeys[i]))}`;
          }
          // 这边图层地址要加上'&f=json'，表示信息以json格式返回回来
          validLayerInfoUrl = `${layerUrl}&f=json`;
        } else {
          // 这边图层地址要加上'&f=json'，表示信息以json格式返回回来
          validLayerInfoUrl = `${layerUrl}?f=json`;
        }
        return new Promise((resolve) => {
          if (layerInfo.checkValid === true) {
            this.checkLayerInfoValid(validLayerInfoUrl).then(validResult => {
              resolve(validResult);
            });
          } else {
            resolve(true);
          }
        }).then(validResult => {
          let esriLayersArcGISTiledMapServiceLayer;
          if (validResult) {
            esriLayersArcGISTiledMapServiceLayer = new EsriLayersArcGISTiledMapServiceLayer(layerUrl, {
              id: layerInfo.id
            });
            map.addLayer(esriLayersArcGISTiledMapServiceLayer, index);
            // 调整右下角的权属单位、
            // this.changeCopyRight();
          }
          return {
            map: map,
            layer: esriLayersArcGISTiledMapServiceLayer,
          };
        });

      });
    };
  /** 加载geoJson类型的矢量切片图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapVectorGeoJsonTiledLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadVectorGeoJsonTiledLayer =
    function (map, layerInfo, index) {
      return new Promise((resolve, reject) => {
        this.layerService.showWarnInfo('vector geoJson tiled layer加载未实现！');
        resolve({
          map: map
        });
      });
    };
  /** 加载矢量切片图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapVectorTiledLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadVectorTiledLayer = function (map, layerInfo, index) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('vector tiled layer加载未实现！');
      resolve({
        map: map
      });
    });
  };
  /** 加载wms类型的图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriWMSLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.loadWMSLayer = function (map, layerInfo, index) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('wms layer加载未实现！');
      resolve({
        map: map
      });
    });
  };
  /** 在地图中测量面积
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.measureArea = function (map) {
    const modules = [
      'esri/graphic',
      'esri/toolbars/draw',
      'esri/symbols/Font',
      'esri/Color',
      'esri/symbols/SimpleLineSymbol',
      'esri/geometry/geometryEngine',
      'esri/symbols/TextSymbol',
      'esri/symbols/SimpleFillSymbol',
    ];
    return this.loadModules(modules).then(
      ([Graphic, Draw, Font, Color, SimpleLineSymbol, geometryEngine, TextSymbol, SimpleFillSymbol]) => {
        //  drawTool = this.deactivateDrawTool(drawTool);
        const drawTool = new Draw(map);
        map.setMapCursor('crosshair');
        drawTool.activate(Draw.POLYGON);

        return new Promise((resolve, reject) => {
          drawTool.onDrawComplete = (event) => {
            const point = event.geometry.getCentroid();
            this.measureLengthOrArea(
              map,
              geometryEngine,
              Font,
              TextSymbol,
              Color,
              Graphic,
              event.geometry,
              point,
              false,
              drawTool,
            ).then(callback => {
              const callbackMap = callback['map'],
                value = callback['value'] + '平方米';
              resolve({
                map: callbackMap,
                // drawTool: drawTool,
                value: value
              });
            });
          };
        });

      }
    );
  };
  /** 在地图中测量距离
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.measureLength = function (map) {
    const modules = [
      'esri/layers/GraphicsLayer',
      'esri/graphic',
      'esri/toolbars/draw',
      'esri/symbols/Font',
      'esri/Color',
      'esri/symbols/SimpleLineSymbol',
      'esri/geometry/geometryEngine',
      'esri/symbols/TextSymbol',
      'esri/geometry/Point',
    ];
    return this.loadModules(modules).then(
      ([GraphicsLayer, Graphic, Draw, Font, Color, SimpleLineSymbol, geometryEngine, TextSymbol, Point, ]) => {
        // drawTool = this.deactivateDrawTool(drawTool);
        const drawTool = new Draw(map);
        map.setMapCursor('crosshair');
        drawTool.activate(Draw.POLYLINE);

        return new Promise((resolve, reject) => {
          drawTool.onDrawComplete = (event) => {
            const lastPoint = event.geometry.paths[0][event.geometry.paths[0].length - 1],
              point = new Point(lastPoint[0], lastPoint[1], event.geometry.spatialReference);
            this.measureLengthOrArea(
              map,
              geometryEngine,
              Font,
              TextSymbol,
              Color,
              Graphic,
              event.geometry,
              point,
              true,
              drawTool,
            ).then(callback => {
              const callbackMap = callback['map'],
                value = callback['value'] + '米';
              resolve({
                map: callbackMap,
                // drawTool: drawTool,
                value: value
              });
            });
          };
        });
      }
    );
  };
  /** 测量距离或者面积
   *
   *
   * @params {*} map
   * @params {*} geometryEngine
   * @params {*} Font
   * @params {*} TextSymbol
   * @params {*} Color
   * @params {*} Graphic
   * @params {*} geometry
   * @params {*} point
   * @params {boolean} measureLength
   * @params {*} drawTool
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.measureLengthOrArea = function (map, geometryEngine, Font, TextSymbol, Color, Graphic, geometry, point, measureLength, drawTool) {
    return this.getSymbol(geometry.type, 'measure_symbol').then((symbol) => {
      drawTool.deactivate();
      map.setMapCursor('default');
      map.graphics.add(new Graphic(geometry, symbol));
      let value, unit;
      if (measureLength) {
        value = geometryEngine.planarLength(geometry, 'meters').toFixed(2);
        unit = '米';
      } else {
        value = geometryEngine.planarArea(geometry, 'square-meters').toFixed(2);
        unit = '平方米';
      }
      const font = new Font('12px', Font.STYLE_NORMAL, Font.letIANT_NORMAL),
        textSymbol = new TextSymbol(`${value}${unit}`, font, new Color([0, 0, 0])),
        labelGraphic = new Graphic(point, textSymbol);

      map.graphics.add(labelGraphic);
      return {
        map: map,
        value: value,
      };
    });
  };
  /** 调整地图上指定图层的层级
   *
   *
   * @params {*} map map对象
   * @params {string} layerId 图层id
   * @params {string} level 层级
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.moveLayerLevel = function (map, layerId, level) {
    const mapLayer = map.getLayer(layerId);
    if (mapLayer) {
      let index = -1;
      switch (level) {
        case '~+1':
          index = map.layerIds.length - 1;
          break;
        case '~-1':
          index = 1;
          break;
        case '+1':
          index = _.indexOf(map.layerIds, layerId) + 1;
          break;
        case '-1':
          index = _.indexOf(map.layerIds, layerId) - 1;
          break;
      }
      map.reorderLayer(mapLayer, index);
    } else {
      console.warn('未找到此图层');
    }
    return map;
  };
  /** 移除地图上指定的图层
   *
   *
   * @params {*} map map对象
   * @params {string} layerId 图层id
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.removeMapLayer = function (map, layerId) {
    const mapLayer = map.getLayer(layerId);
    if (mapLayer) {
      map.removeLayer(mapLayer);
    } else {
      console.warn('未找到此图层！layerId:' + layerId);
    }
  };
  /** 移除地图上所有的图层
   *
   *
   * @params {*} map map对象
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.removeMapAllLayer = function (map) {
    for (let i = 0; i < map.layerIds.length; i++) {
      const layerId = map.layerIds[i];
      this.removeMapLayer(map, layerId);
    }
  };
  /** 在地图上选择圆形区域
   *
   *
   * @params {*} map
   * @params {*} drawTool
   * @returnses {Promise<Object>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.selectCircle = function (map) {
    return this.drawGraphics(map, V_SIPSD_MAP_ARCGIS_DRAW_TYPE.CIRCLE).then(callback => {
      map = callback['map'];
      // drawTool = callback['drawTool'];
      const geometry = this.convertGeometry2Wkt(callback['geometry']);
      return {
        result: true,
        map: map,
        // drawTool: drawTool,
        value: geometry
      };
    });
  };
  /** 在地图上选择矩形区域
   *
   *
   * @params {*} map
   * @params {*} drawTool
   * @returnses {Promise<Object>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.selectRectangle = function (map) {
    return this.drawGraphics(map, V_SIPSD_MAP_ARCGIS_DRAW_TYPE.RECTANGLE).then(callback => {
      map = callback['map'];
      // drawTool = callback['drawTool'];
      const geometry = this.convertGeometry2Wkt(callback['geometry']);
      return {
        result: true,
        map: map,
        // drawTool: drawTool,
        value: geometry
      };
    });
  };
  /** 在地图上选择多边形区域
   *
   *
   * @params {*} map
   * @params {*} drawTool
   * @returnses {Promise<Object>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.selectPolygon = function (map) {
    return this.drawGraphics(map, V_SIPSD_MAP_ARCGIS_DRAW_TYPE.POLYGON).then(callback => {
      map = callback['map'];
      // drawTool = callback['drawTool'];
      const geometry = this.convertGeometry2Wkt(callback['geometry']);
      return {
        result: true,
        map: map,
        // drawTool: drawTool,
        value: geometry
      };
    });
  };
  /** 设置地图初始化范围
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapBaseInfo} mapInfo 地图信息
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.setInitExtent = function (map, mapInfo) {
    if (map.spatialReference.wkt.indexOf(mapInfo.spatialReference) !== -1 ||
      map.spatialReference.wkt.toUpperCase() === mapInfo.spatialReference.toUpperCase()) {
      const extentInfo = mapInfo.extent;
      return this.setMapExtent(map, extentInfo).then(callback => {
        return {
          result: true,
          map: callback
        };
      });
    } else {
      return new Promise((resolve, reject) => {
        this.layerService.showInfo({
          content: '图层坐标系不一致',
          icon: 2,
          yes: (index, layero) => {
            this.layerService.hideInfo(index);
            resolve({
              result: false,
              map: map
            });
          },
          cancel: (index, layero) => {
            this.layerService.hideInfo(index);
            resolve({
              result: false,
              map: map
            });
          }
        });
      });
    }
  };
  /** 设置地图上指定图层的透明度
   *
   *
   * @params {*} map map对象
   * @params {string} layerId 图层id
   * @params {number} opacity 透明度
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.setLayerOpacity = function (map, layerId, opacity) {
    const mapLayer = map.getLayer(layerId);
    if (mapLayer) {
      mapLayer.setOpacity(opacity);
      mapLayer.refresh();
    } else {
      console.warn('未找到此图层！layerId:' + layerId);
    }
  };
  /** 设置地图范围
   *
   *
   * @params {*} map map对象
   * @params {Object} extent 范围信息
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.setMapExtent = function (map, extent) {
    const modules = ['esri/geometry/Extent'];
    return this.loadModules(modules).then(([EsriGeometryExtent]) => {
      map.setExtent(new EsriGeometryExtent(
        extent['xmin'],
        extent['ymin'],
        extent['xmax'],
        extent['ymax'],
        map.spatialReference
      ));
      return map;
    });
  };
  /** 设置鼠标移动地图
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Object}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.setPan = function (map) {
    map.setMapCursor('default');
    map.enablePan();
    return {
      map: map,
    };
  };
  /** 设置地图缩放程度
   *
   *
   * @params {*} map map对象
   * @params {boolean} zoomIn 是否放大，true是放大，false是缩小
   * @returnses {Promise<any>}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.setZoom = function (map, zoomIn) {
    const currentZoom = map.getZoom(),
      zoomLevel = zoomIn ? 1 : -1,
      zoom = currentZoom + zoomLevel;
    const result = map.setZoom(zoom > -1 ? zoom : 0);
    return result.then((data) => {
      return {
        result: result.isFulfilled(),
        map: map,
      };
    });
  };
  /** 显示某个地图气泡框内容
   *
   *
   * @params {*} map
   * @params {*} popUpGeometry
   * @params {string} popUpTitle
   * @params {boolean} zoom
   * @returnses {*}
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.showMapPopUp = function (map, popUpGeometry, popUpTitle, zoom) {
    let popGeoCenter;
    if (popUpGeometry.type === 'polygon' || popUpGeometry.type === 'polyline') {
      popGeoCenter = popUpGeometry.getExtent().getCenter();
    } else {
      popGeoCenter = popUpGeometry;
    }

    const screenPoint = map.toScreen(popGeoCenter);
    map.infoWindow.setTitle(popUpTitle);
    map.infoWindow.show(screenPoint, map.getInfoWindowAnchor(screenPoint));

    if (zoom) {
      map.centerAndZoom(popGeoCenter, map.getMaxZoom() - 2);
    } else {
      map.centerAt(popGeoCenter);
    }

    // 点击气泡框右上角的x按钮时触发的事件
    map.infoWindow.on('hide', () => {
      // this.clearHightlightGraphics(map);
    });
    return map;
  };
  /** 展示地图气泡框
   *
   *
   * @params {*} map
   * @params {ArcgisMapPopUps} popUps
   * @params {number} mapPopUpSelectedIndex
   * @returnses
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.showMapPopUps =
    function (map, popUps, mapPopUpSelectedIndex) {
      // 不管是显示i查询的结果还是显示普通查询的定位结果
      // 都需要先将原来的结果图层（如果有的话）给去除掉
      // 否则会有很多无用的定位符号数据。如果这些不处理掉
      // 的话，在监听点击定位符号事件时，有多少无用数据，
      // 就会多触发几次，造成很不好的影响
      this.clearPopUps(map);
      const popUpList = popUps.popUpList;
      if (!popUpList || popUpList.length <= 0) {
        return new Promise((resolve, reject) => {
          resolve({
            result: false,
            type: popUps.type,
          });
        });
      }
      if ('iQuery' === popUps.type) {
        const popUp = popUpList[mapPopUpSelectedIndex];
        if (popUp) {
          const popGeometry = this.convertWkt2Geometry(map, popUp.geometry),
            hasPrev = mapPopUpSelectedIndex === 0,
            hasNext = mapPopUpSelectedIndex === popUpList.length - 1;
          popUp.hasPrev = !hasPrev;
          popUp.hasNext = !hasNext;
          popUp.index = mapPopUpSelectedIndex;
          popUp.total = popUpList.length;
          const modules = [
            'esri/layers/GraphicsLayer',
            'esri/graphic',
            'esri/Color',
            'esri/symbols/SimpleMarkerSymbol',
            'esri/symbols/SimpleLineSymbol',
            'esri/symbols/SimpleFillSymbol',
            'esri/symbols/PictureMarkerSymbol',
          ];
          return this.loadModules(modules).then(([
            GraphicsLayer,
            Graphic,
            Color,
            SimpleMarkerSymbol,
            SimpleLineSymbol,
            SimpleFillSymbol,
          ]) => {
            const highlightAreaConstructorClass = {
              GraphicsLayer: GraphicsLayer,
              Graphic: Graphic,
              Color: Color,
              SimpleMarkerSymbol: SimpleMarkerSymbol,
              SimpleLineSymbol: SimpleLineSymbol,
              SimpleFillSymbol: SimpleFillSymbol,
            };
            this.createHighlightAreaLayer(map, [popUp], highlightAreaConstructorClass);
            this.showMapPopUp(map, popGeometry, popUp.title, popUps.zoom);
            return {
              result: true,
              type: popUps.type,
              popUp: popUp,
              mapPopUpSelectedIndex: mapPopUpSelectedIndex,
            };
          });
        } else {
          console.warn(`第${mapPopUpSelectedIndex + 1}个气泡框内容不存在，请检查！`);
          return new Promise((resolve, reject) => {
            resolve({
              result: false,
              type: popUps.type,
              mapPopUpSelectedIndex: mapPopUpSelectedIndex,
            });
          });
        }
      } else {
        const modules = [
          'esri/layers/GraphicsLayer',
          'esri/graphic',
          'esri/Color',
          'esri/symbols/SimpleMarkerSymbol',
          'esri/symbols/SimpleLineSymbol',
          'esri/symbols/SimpleFillSymbol',
          'esri/symbols/PictureMarkerSymbol',
        ];
        return this.loadModules(modules).then(([
          GraphicsLayer,
          Graphic,
          Color,
          SimpleMarkerSymbol,
          SimpleLineSymbol,
          SimpleFillSymbol,
          PictureMarkerSymbol,
        ]) => {
          const highlightAreaConstructorClass = {
              GraphicsLayer: GraphicsLayer,
              Graphic: Graphic,
              Color: Color,
              SimpleMarkerSymbol: SimpleMarkerSymbol,
              SimpleLineSymbol: SimpleLineSymbol,
              SimpleFillSymbol: SimpleFillSymbol,
            },
            iconMarkerConstructorClass = {
              GraphicsLayer: GraphicsLayer,
              Graphic: Graphic,
              PictureMarkerSymbol: PictureMarkerSymbol,
            };
          this.createHighlightAreaLayer(map, popUpList, highlightAreaConstructorClass);
          const iconMarkerLayerId = ICON_MARKER_LAYER_ID;
          if ('nullIndexPopUps' === popUps.type) {
            this.createNullIndexIconMarkersLayer(map, iconMarkerLayerId, popUpList, iconMarkerConstructorClass);
          } else {
            this.createIndexIconMarkersLayer(map, iconMarkerLayerId, popUpList, iconMarkerConstructorClass);
          }
          const popUp = popUpList[mapPopUpSelectedIndex];
          this.showMapPopUp(map, this.convertWkt2Geometry(map, popUp.geometry), popUp.title, popUps.zoom);
          return {
            result: true,
            type: popUps.type,
            popUp: popUp,
            mapPopUpSelectedIndex: mapPopUpSelectedIndex,
          }
        });

      }
    };
  /** 在图标标识图层中，根据图标标识索引置顶相应的图标标识
   *
   *
   * @params {*} map
   * @params {number} index
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.topIconMarkerItemByIndexInIconMarkersLayer = function (map, index) {
    const iconMarkerItem = this.getMarkerItemFromIconMarkersLayer(map, index);
    this.topIconMarkerItemInIconMarkersLayer(map, iconMarkerItem);
  };
  /** 置顶图标标识图层中的图标标识
   *
   *
   * @params {*} map 地图实例
   * @params {*} iconMarkerItem 图标标识
   * @memberof VSipsdMapArcgisService
   */
  VSipsdMapArcgisService.prototype.topIconMarkerItemInIconMarkersLayer = function (map, iconMarkerItem) {
    const iconMarkersLayer = map.getLayer(ICON_MARKER_LAYER_ID);
    if (iconMarkersLayer) {
      // 先移除，再加入，达到置顶的效果
      iconMarkersLayer.remove(iconMarkerItem);
      iconMarkersLayer.add(iconMarkerItem);
    }
  };
  return VSipsdMapArcgisService;
}());
export {
  VSipsdMapArcgisService
};
