import TileLayer from 'ol/layer/Tile';
import ImageLayer from 'ol/layer/Image';
import VectorLayer from 'ol/layer/Vector';
import VectorTileLayer from 'ol/layer/VectorTile';
import Heatmap from 'ol/layer/Heatmap';
import VectorSource from 'ol/source/Vector';
import VectorTile from 'ol/source/VectorTile';
import Style from 'ol/style/Style';
import Fill from 'ol/style/Fill';
import Stroke from 'ol/style/Stroke';
import CircleStyle from 'ol/style/Circle';
import Icon from 'ol/style/Icon';
import Text from 'ol/style/Text';
import Draw, {
  createBox as createBox_ol_interaction_Draw,
} from 'ol/interaction/Draw';
import Polygon, {
  fromCircle as fromCircle_ol_geom_Polygon,
} from 'ol/geom/Polygon';
import LineString from 'ol/geom/LineString';
import Point from 'ol/geom/Point';
import Circle from 'ol/geom/Circle';
import {
  unByKey as unByKey_ol_Observable
} from 'ol/Observable';
import GeoJSON from 'ol/format/GeoJSON';
import WKT from 'ol/format/WKT';
import Overlay from 'ol/Overlay';
import Map from 'ol/Map';
import ScaleLine from 'ol/control/ScaleLine';
import Feature from 'ol/Feature';
import {
  getArea as getArea_ol_sphere,
  getLength as getLength_ol_sphere,
} from 'ol/sphere';
import {
  fromLonLat as fromLonLat_ol_proj,
  toLonLat as toLonLat_ol_proj,
  get as get_ol_proj
} from 'ol/proj';
import {
  getCenter as getCenter_ol_extent
} from 'ol/extent';
import ImageArcGISRest from 'ol/source/ImageArcGISRest';
import XYZ from 'ol/source/XYZ';
import TileGrid from 'ol/tilegrid/TileGrid';
import View from 'ol/View';
import MVT from 'ol/format/MVT';
import {
  asArray as asArray_ol_Color
} from 'ol/color';
import {
  DataRequestService,
} from './../../v-sipsd-tool-services';
import {
  VSipsdLayuiLayerService
} from './../../v-sipsd-layui-layer';
import {
  jQuery,
  _,
} from './../../3rd.party';
import TianMap from '../../../sipsd-map-3rd-party/ol.source.TianMap.js';
import {
  icon_dibiao_null,
  icon_dibiao_arrs
} from './../icon_dibiao_base64';
import {
  V_SIPSD_MAP_OPENLAYER_DRAW_TYPE,
  V_SIPSD_MAP_TYPE,
  HIGHLIGHT_AREA_LAYER_ID,
  ICON_MARKER_LAYER_ID,
  POPUP_LAYER_ID,
  HIGHLIGHT_AREA_LAYER_INDEX,
  ICON_MARKER_LAYER_INDEX,
} from './../ng-sipsd-map.class';
const VSipsdMapOpenlayerService = (function () {
  function VSipsdMapOpenlayerService() {
    this.dataRequest = new DataRequestService();
    this.layerService = new VSipsdLayuiLayerService();
  }
  /** 增加相互作用的量
   *
   *
   * @params {*} map
   * @params {V_SIPSD_MAP_OPENLAYER_DRAW_TYPE} type
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.addInteraction = function (map, type) {
    this.setPan(map);
    return new Promise((resolve, reject) => {
      const source = new VectorSource(),
        vector = new VectorLayer({
          source: source,
          style: new Style({
            fill: new Fill({
              color: 'rgba(255, 255, 255, 0.2)'
            }),
            stroke: new Stroke({
              color: '#ffcc33',
              width: 2
            }),
            image: new CircleStyle({
              radius: 7,
              fill: new Fill({
                color: '#ffcc33'
              })
            })
          })
        });
      vector.set('id', 'measure-vector');
      map.addLayer(vector);

      const measureTooltipObj = this.createMeasureTooltip(map),
        helpTooltipObj = this.createHelpTooltip(measureTooltipObj['map']);
      map = helpTooltipObj['map'];
      const measureTooltipElement = measureTooltipObj['element'],
        helpTooltipElement = helpTooltipObj['element'];
      const measureTooltip = map.getOverlayById('measureTooltip'),
        helpTooltip = map.getOverlayById('helpTooltip');

      const draw = new Draw({
        source: source,
        type: type.toString(),
        style: new Style({
          fill: new Fill({
            color: 'rgba(255, 255, 255, 0.2)'
          }),
          stroke: new Stroke({
            color: 'rgba(0, 0, 0, 0.5)',
            lineDash: [10, 10],
            width: 2
          }),
          image: new CircleStyle({
            radius: 5,
            stroke: new Stroke({
              color: 'rgba(0, 0, 0, 0.7)'
            }),
            fill: new Fill({
              color: 'rgba(255, 255, 255, 0.2)'
            })
          })
        })
      });
      map.addInteraction(draw);
      map.getViewport().addEventListener('mouseout', (mouseoutEvent) => {
        helpTooltipElement.classList.add('hidden');
      });

      let listener, sketch, output;
      // 开始画图
      draw.on('drawstart', (drawstartEvent) => {
        sketch = drawstartEvent.feature;
        let tooltipCoord = drawstartEvent.coordinate;

        listener = sketch.getGeometry().on('change', (changeEvent) => {
          const geomData = changeEvent.target;
          // 判断是进行距离的测量还是面积的测量
          if (geomData instanceof Polygon) {
            // 转成面积值
            output = this.formatArea(geomData);
            // 获取坐标
            tooltipCoord = geomData.getInteriorPoint().getCoordinates();
          } else if (geomData instanceof LineString) {
            // 转成长度值
            output = this.formatLength(geomData);
            // 获取坐标
            tooltipCoord = geomData.getLastCoordinate();
          }
          measureTooltipElement.innerHTML = output;

          measureTooltip.setPosition(tooltipCoord);
        });

      });
      // 结束画图
      draw.on('drawend', (drawendEvent) => {
        measureTooltipElement.className = 'tooltip tooltip-static';
        measureTooltip.setOffset([0, -7]);

        unByKey_ol_Observable(listener);
        setTimeout(() => {
          // 将画图实例对象移除销毁
          map.removeInteraction(draw);
        });
        // 移除提示信息
        map.removeOverlay(helpTooltip);
        resolve({
          map: map,
          value: output
        });
      });

      // 鼠标点移动时
      map.on('pointermove', (pointermoveEvent) => {
        if (pointermoveEvent.dragging) {
          return;
        }

        let helpMsg = '请点击鼠标左键开始测量...';

        if (sketch) {
          const geomData = sketch.getGeometry();
          if (geomData instanceof Polygon) {
            helpMsg = '请单击鼠标左键以绘制区域，双击以完成面积的测量！';
          } else if (geomData instanceof LineString) {
            helpMsg = '请单击鼠标左键以调整方向，双击以完成距离的测量！';
          }
        }

        helpTooltipElement.innerHTML = helpMsg;
        helpTooltip.setPosition(pointermoveEvent.coordinate);
        helpTooltipElement.classList.remove('hidden');
      });
    });

  };

  /** 增加矢量图层
   *
   *
   * @params {*} map
   * @params {string} layerId
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.addVectorLayer = function (map, layerId) {
    const source = new VectorSource();
    const vector = new VectorLayer({
      source: source
    });

    vector.set('id', layerId);
    map.addLayer(vector);
    return vector;

  };

  /** 更改基础图层
   *
   *
   * @params {*} map
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.changeMapBaseLayer = function (map, layerInfo) {
    const baseLayer = this.getBaseLayer(map);
    if (baseLayer) {
      this.removeMapLayer(map, baseLayer.get('id'));
    }

    return this.loadMapLayer(map, layerInfo, 0, true);
  };

  /** 检查返回的图层信息是否有效
   *
   *
   * @params {Object} layerInfo
   * @returnses {boolean}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.checkLayerInfoValid = function (layerInfo) {
    // 网关的异常状态码
    // if (40101 === layerInfo['status'] ||
    //   40102 === layerInfo['status'] ||
    //   40131 === layerInfo['status'] ||
    //   40331 === layerInfo['status']) {
    //   const content = layerInfo['message'];
    //   this.layerService.showErrorInfo(content);
    //   return false;
    // }
    if (!layerInfo) {
      return false;
    }

    if (layerInfo['status']) {
      const content = JSON.stringify(layerInfo);
      this.layerService.showErrorInfo(content);
      return false;
    }
    // 原来平台的地图服务报错的情况的处理，格式：
    // {Status: "500.3.1", Des: "令牌验证错误!", Msg: "令牌[token]验证错误，请检查令牌是否正确!"}
    if (layerInfo['Status']) {
      const content = layerInfo['Msg'];
      this.layerService.showErrorInfo(content);
      return false;
    }
    return true;
  };

  /** 检查图层的类型
   *
   *
   * @params {*} layerInstance
   * @returnses {string}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.checkLayerType = function (layerInstance) {
    if (layerInstance instanceof TileLayer) {
      return 'Tile';
    } else if (layerInstance instanceof ImageLayer) {
      return 'Image';
    } else if (layerInstance instanceof VectorLayer) {
      return 'Vector';
    } else if (layerInstance instanceof VectorTileLayer) {
      return 'VectorTile';
    } else if (layerInstance instanceof Heatmap) {
      return 'Heatmap';
    } else {
      return null;
    }
  };

  /** 检查地图服务地址是否是合法有效的
   *
   *
   * @params {string} layerUrl 地图服务地址
   * @returnses {boolean} 合法有效的地址返回true，不合法无效的返回false
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.checkLayerUrlValid = function (layerUrl) {
    if (layerUrl === undefined || layerUrl === null || '' === layerUrl.trim()) {
      this.layerService.showWarnInfo(`“${layerUrl}”为无效的地图服务地址，无法加载该图层！`);
      return false;
    }
    return true;
  };

  /** 清除地图上的图形信息
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Object}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.clearGraphics = function (map, ) {
    if ((typeof map.getLayers) === 'function') {
      const layers = map.getLayers();
      for (let i = layers.getLength(); i >= 0; i--) {
        const layerInstance = layers.item(i);
        if (layerInstance instanceof VectorLayer) {
          map.removeLayer(layerInstance);
        }
      }
      map = this.clearOverlay(map)['map'];
      map = this.setPan(map)['map'];
    }
    return {
      map: map,
    };
  };

  /** 清除覆盖图层
   *
   *
   * @params {*} map
   * @params {string} [overlayId]
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.clearOverlay = function (map, overlayId) {
    if (overlayId) {
      map.getOverlayById(overlayId).clear();
    } else {
      map.getOverlays().clear();
    }
    return {
      map: map
    };
  };

  /** 清除气泡
   *
   *
   * @params {*} map
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.clearPopUps = function (map) {
    // 将定位符号图层删除
    const symbolLayer = this.getLayer(map, ICON_MARKER_LAYER_ID);
    if (symbolLayer) {
      map.removeLayer(symbolLayer);
    }
    // 将高亮区域部分图层删除
    const popUpItemHighlightLayerId = HIGHLIGHT_AREA_LAYER_ID;
    this.removeMapLayer(map, popUpItemHighlightLayerId);
    // 将含有气泡框的覆盖物删除
    const overlays = map.getOverlays();
    if (overlays) {
      overlays.forEach((item, index) => {
        if (item) {
          const id = item.getId();
          if (id.indexOf('-popUp') > -1) {
            map.removeOverlay(item);
          }
        }
      });
    }
  };

  /** 将圆形geometry转成多边形geometry
   *
   *
   * @params {*} circleGeometry
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertCircleGeometry2PolygonGeometry = function (circleGeometry) {
    if (circleGeometry instanceof Circle) {
      circleGeometry = fromCircle_ol_geom_Polygon(circleGeometry);
    }
    return circleGeometry;
  };

  /** 将坐标装成geometry几何数据类型
   *
   *
   * @params {any} type
   * @params {any} coordinates
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertCoordinates2Geometry = function (type, coordinates) {
    switch (type) {
      case V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POINT:
        return new Point(coordinates);
      case V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYLINE:
        return new LineString(coordinates);
      case V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYGON:
        return new Polygon(coordinates);
    }
  };

  /** 将geometry的各个坐标点转成4326投影下的经纬度坐标点
   *
   *
   * @params {*} geometry
   * @params {*} [opt_projection]
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertGeometryCoordinates2LonLat = function (geometry, opt_projection) {
    geometry = geometry.clone();
    if (opt_projection === undefined || opt_projection === null) {
      opt_projection = 'EPSG:3857';
    }
    const geometryType = geometry.getType();
    if ('Point' === geometryType) {
      const coordinates = toLonLat_ol_proj(geometry.getCoordinates(), opt_projection);
      geometry.setCoordinates(coordinates);
    } else if ('LineString' === geometryType) {
      const coordinates = geometry.getCoordinates(),
        newCoordinates = [];
      for (const item of coordinates) {
        newCoordinates.push(toLonLat_ol_proj(item, opt_projection));
      }
      geometry.setCoordinates(newCoordinates);
    } else if ('Polygon' === geometryType) {
      const coordinates = geometry.getCoordinates(),
        newCoordinates = [];
      for (const coordinate of coordinates) {
        const newCoordinate = [];
        for (const item of coordinate) {
          newCoordinate.push(toLonLat_ol_proj(item, opt_projection));
        }
        newCoordinates.push(newCoordinate);
      }
      geometry.setCoordinates(newCoordinates);
    } else if ('Circle' === geometryType) {

    }
    return geometry;
  };
  /** 将geometry的各个坐标点从经纬度转成3587投影下的坐标
   *
   *
   * @params {*} geometry
   * @params {*} [opt_projection]
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertGeometryCoordinatesFromLonLat = function (geometry, opt_projection) {
    geometry = geometry.clone();
    if (opt_projection === undefined || opt_projection === null) {
      opt_projection = 'EPSG:3857';
    }
    const geometryType = geometry.getType();
    if ('Point' === geometryType) {
      const coordinates = fromLonLat_ol_proj(geometry.getCoordinates(), opt_projection);
      geometry.setCoordinates(coordinates);
    } else if ('LineString' === geometryType) {
      const coordinates = geometry.getCoordinates(),
        newCoordinates = [];
      for (const item of coordinates) {
        newCoordinates.push(fromLonLat_ol_proj(item, opt_projection));
      }
      geometry.setCoordinates(newCoordinates);
    } else if ('Polygon' === geometryType) {
      const coordinates = geometry.getCoordinates(),
        newCoordinates = [];
      for (const coordinate of coordinates) {
        const newCoordinate = [];
        for (const item of coordinate) {
          newCoordinate.push(fromLonLat_ol_proj(item, opt_projection));
        }
        newCoordinates.push(newCoordinate);
      }
      geometry.setCoordinates(newCoordinates);
    } else if ('Circle' === geometryType) {

    }
    return geometry;
  };

  /** 将geometry的json转成features的
   *
   *
   * @params {any} geojsonObject
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertGeojson2Features = function (geojsonObject) {
    return (new GeoJSON()).readFeatures(geojsonObject);

  };

  /** 将geometry转换成wkt
   *
   *
   * @params {*} geometry
   * @returnses {string}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertGeometry2Wkt = function (geometry) {
    const format = new WKT();
    return format.writeGeometry(geometry);
  };

  /** 将wkt转换成geometry
   *
   *
   * @params {string} wktStr
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.convertWkt2Geometry = function (wktStr) {
    const format = new WKT();
    return format.readGeometry(wktStr);
  };

  /** 创建双地图
   *
   *
   * @params {string} domNodeId
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createDoubleMap = function (domNodeId) {
    return this.createMap(domNodeId);
  };

  /** 创建帮助提示条
   *
   *
   * @params {*} map
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createHelpTooltip = function (map, ) {
    const helpTooltipElement = document.createElement('div');
    helpTooltipElement.className = 'tooltip hidden';
    let helpTooltip = new Overlay({
      id: 'helpTooltip',
      element: helpTooltipElement,
      offset: [15, 0],
      positioning: 'center-left'
    });
    map.addOverlay(helpTooltip);
    return {
      map: map,
      element: helpTooltipElement
    };
  };

  /** 创建高亮区域项
   *
   *
   * @params {*} geometry
   * @params {*} geometryStyle
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createHighlightAreaItem = function (
    geometry,
    geometryStyle,
    constructorClass,
  ) {
    let style;
    const defaultStyle = {
        fill: {
          color: '#3199ff',
          opacity: 0.5
        },
        border: {
          color: '#3199ff',
          width: 2
        }
      },
      FeatureClass = constructorClass['Feature'],
      asArray_ol_Color = constructorClass['Color2Array'];

    if (!geometryStyle) {
      geometryStyle = defaultStyle;
    }
    if (geometryStyle.customStyle) {
      style = geometryStyle.customStyle;
    } else {
      if (!geometryStyle.fill) {
        geometryStyle.fill = defaultStyle.fill;
      } else {
        if (!geometryStyle.fill.color) {
          geometryStyle.fill.color = defaultStyle.fill.color;
        }
        if (geometryStyle.fill.opacity === undefined || geometryStyle.fill.opacity === null) {
          geometryStyle.fill.opacity = defaultStyle.fill.opacity;
        }
      }
      if (!geometryStyle.border) {
        geometryStyle.border = defaultStyle.border;
      } else {
        if (!geometryStyle.border.color) {
          geometryStyle.border.color = defaultStyle.border.color;
        }
        if (geometryStyle.border.width === undefined || geometryStyle.border.width === null) {
          geometryStyle.border.width = defaultStyle.border.width;
        }
      }
      let fillColor = asArray_ol_Color(geometryStyle.fill.color),
        borderColor = asArray_ol_Color(geometryStyle.border.color);
      fillColor[3] = geometryStyle.fill.opacity;
      style = new Style({
        fill: new Fill({
          color: fillColor
        }),
        stroke: new Stroke({
          color: borderColor,
          width: geometryStyle.border.width
        }),
        image: new CircleStyle({
          // radius主要是在图形是个点时，有个点状图形
          radius: geometryStyle.border.width,
          stroke: new Stroke({
            color: fillColor
          }),
          fill: new Fill({
            color: fillColor
          })
        })
      });

    }

    const areaItem = new FeatureClass({
      geometry: geometry,
    });
    areaItem.setStyle(style);
    return areaItem;
  };

  /** 创建高亮区域图层
   *
   *
   * @params {*} map
   * @params {Object[]} highlightAreaList
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createHighlightAreaLayer = function (
    map,
    highlightAreaList,
    constructorClass,
  ) {
    const layerId = HIGHLIGHT_AREA_LAYER_ID,
      VectorLayerClass = constructorClass['VectorLayer'],
      VectorSourceClass = constructorClass['VectorSource'],
      areaList = [],
      isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);

    let highlightAreaLayer = this.getLayer(map, layerId);
    if (highlightAreaLayer) {
      this.removeMapLayer(map, layerId);
    }

    for (let i = 0; i < highlightAreaList.length; i++) {
      const item = highlightAreaList[i];
      let geometry = this.convertWkt2Geometry(item['geometry']);
      if (isEPSG4326) {
        geometry = this.convertGeometryCoordinatesFromLonLat(geometry);
      }
      const areaItem = this.createHighlightAreaItem(geometry, item['geometryStyle'], constructorClass);
      areaList.push(areaItem);
    }
    highlightAreaLayer = new VectorLayerClass({
      source: new VectorSourceClass({
        features: areaList
      })
    });
    highlightAreaLayer.set('id', layerId);
    highlightAreaLayer.setZIndex(HIGHLIGHT_AREA_LAYER_INDEX);

    map.addLayer(highlightAreaLayer);

    return {
      map: map,
      layer: highlightAreaLayer,
    };
  };

  /** 创建带有geometry的索引图标标识
   *
   *
   * @params {number} index
   * @params {*} geometry
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createIndexIconMarkerItemWidthGeometry = function (index, geometry, constructorClass, ) {
    const iconPath = icon_dibiao_arrs[index],
      FeatureClass = constructorClass['Feature'],
      PointClass = constructorClass['Point'],
      StyleClass = constructorClass['Style'],
      IconClass = constructorClass['Icon'];
    const iconFeature = new FeatureClass({
      geometry: new PointClass(geometry),
    });
    const iconStyle = new StyleClass({
      image: new IconClass(({
        anchor: [0.49, 1],
        anchorXUnits: 'fraction',
        anchorYUnits: 'fraction',
        src: iconPath
      }))
    });
    iconFeature.setStyle(iconStyle);
    return iconFeature;
  };
  /** 创建索引图标标识的图层
   *
   *
   * @params {*} map 地图实例
   * @params {string} layerId 图层id
   * @params {Object[]} indexIconList 索引图标数组
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createIndexIconMarkersLayer = function (map, layerId, indexIconList, constructorClass, ) {
    let indexIconMarkersLayer = this.getLayer(map, layerId);
    if (indexIconMarkersLayer) {
      this.removeMapLayer(map, layerId);
    }
    const markerList = [],
      isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
    for (let i = 0; i < indexIconList.length; i++) {
      const item = indexIconList[i],
        geometry = this.convertWkt2Geometry(item['geometry']);
      const popGeometryCenter = isEPSG4326 ? fromLonLat_ol_proj(this.getGeometryCenter(geometry)) : this.getGeometryCenter(geometry),
        constructorClass = {
          Feature: Feature,
          Point: Point,
          Style: Style,
          Icon: Icon
        };
      const markerItem = this.createIndexIconMarkerItemWidthGeometry(i, popGeometryCenter, constructorClass);
      markerItem.setId(i);
      markerList.push(markerItem);
    }

    const VectorLayerClass = constructorClass['VectorLayer'],
      VectorSourceClass = constructorClass['VectorSource'];
    indexIconMarkersLayer = new VectorLayerClass({
      source: new VectorSourceClass({
        features: markerList
      })
    });
    indexIconMarkersLayer.set('id', layerId);
    // 将定位地图的图层放在最顶层，防止被其它图层覆盖
    indexIconMarkersLayer.setZIndex(ICON_MARKER_LAYER_INDEX);
    map.addLayer(indexIconMarkersLayer);

    return {
      map: map,
      layer: indexIconMarkersLayer,
    }
  };

  /** 创建map实例
   *
   *
   * @params {string} domNodeId map实例所在的dom节点id
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createMap = function (domNodeId, ) {
    return new Promise((resolve, reject) => {
      setTimeout(() => {
        const map = new Map({
          target: domNodeId,
          logo: false,
          controls: [new ScaleLine()]
          // renderer: ('webgl'),
        });
        resolve({
          map: map
        });
      });
    });
  };

  /** 创建测量提示工具条
   *
   *
   * @params {*} map
   * @returnses {Object}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createMeasureTooltip = function (map, ) {
    const measureTooltipElement = document.createElement('div');
    measureTooltipElement.className = 'tooltip tooltip-measure';
    const measureTooltip = new Overlay({
      id: 'measureTooltip',
      element: measureTooltipElement,
      offset: [0, -15],
      positioning: 'bottom-center'
    });
    map.addOverlay(measureTooltip);
    return {
      map: map,
      element: measureTooltipElement
    };
  };

  /** 创建带有geometry的无索引图标标识
   *
   *
   * @params {*} geometry
   * @params {*} constructorClass 构造类
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createNullIndexIconMarkerItemWidthGeometry = function (geometry, constructorClass, ) {
    const iconPath = icon_dibiao_null,
      FeatureClass = constructorClass['Feature'],
      PointClass = constructorClass['Point'],
      StyleClass = constructorClass['Style'],
      IconClass = constructorClass['Icon'];
    const iconFeature = new FeatureClass({
      geometry: new PointClass(geometry),
    });
    const iconStyle = new StyleClass({
      image: new IconClass(({
        anchor: [0.49, 1],
        anchorXUnits: 'fraction',
        anchorYUnits: 'fraction',
        src: iconPath
      }))
    });
    iconFeature.setStyle(iconStyle);
    return iconFeature;
  };

  /** 创建无索引图标标识的图层
   *
   *
   * @params {*} map
   * @params {string} layerId
   * @params {Object[]} nullIndexIconList
   * @params {*} constructorClass
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createNullIndexIconMarkersLayer = function (map, layerId, nullIndexIconList, constructorClass, ) {
    let nullIndexIconMarkersLayer = this.getLayer(map, layerId);
    if (nullIndexIconMarkersLayer) {
      this.removeMapLayer(map, layerId);
    }
    const markerList = [],
      isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
    for (let i = 0; i < nullIndexIconList.length; i++) {
      const item = nullIndexIconList[i],
        geometry = this.convertWkt2Geometry(item['geometry']);
      const popGeometryCenter = isEPSG4326 ? fromLonLat_ol_proj(this.getGeometryCenter(geometry)) : this.getGeometryCenter(geometry);
      const markerItem = this.createNullIndexIconMarkerItemWidthGeometry(popGeometryCenter, constructorClass);
      markerItem.setId(i);
      markerList.push(markerItem);
    }

    const VectorLayerClass = constructorClass['VectorLayer'],
      VectorSourceClass = constructorClass['VectorSource'];
    nullIndexIconMarkersLayer = new VectorLayerClass({
      source: new VectorSourceClass({
        features: markerList
      })
    });
    nullIndexIconMarkersLayer.set('id', layerId);
    // 将定位地图的图层放在最顶层，防止被其它图层覆盖
    nullIndexIconMarkersLayer.setZIndex(ICON_MARKER_LAYER_INDEX);
    map.addLayer(nullIndexIconMarkersLayer);

    return {
      map: map,
      layer: nullIndexIconMarkersLayer,
    }
  };

  /** 创建地图气泡的覆盖物
   *
   *
   * @params {(string | number)} popUpId
   * @params {*} popUpElement
   * @params {number[]} [coordinates]
   * @params {number[]} [offset]
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createPopUpElementOverlay = function (
    popUpId,
    popUpElement,
    coordinates,
    offset
  ) {
    const popUpElementOverlay = new Overlay({
      id: popUpId,
      element: popUpElement,
      autoPan: true,
      offset: offset ? offset : [18, -113],
      autoPanAnimation: {
        duration: 250
      },
    });
    if (coordinates) {
      popUpElementOverlay.setPosition(coordinates);
    }
    return popUpElementOverlay;
  };

  /** 创建显示气泡框元素所在的区域的图层
   *
   *
   * @params {string} layerId
   * @params {string} wktstr
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createPopUpItemAreaLayer = function (layerId, popUp, map) {
    let geometry = this.convertWkt2Geometry(popUp.geometry),
      style;
    const defaultGeometryStyle = {
      fill: {
        color: '#3199ff',
        opacity: 0.5
      },
      border: {
        color: '#3199ff',
        width: 2
      }
    };
    let geometryStyle = popUp.geometryStyle;
    if (!geometryStyle) {
      geometryStyle = defaultGeometryStyle;
    } else {
      if (geometryStyle.customStyle) {
        style = geometryStyle.customStyle;
      } else {
        if (!geometryStyle.fill) {
          geometryStyle.fill = defaultGeometryStyle.fill;
        } else {
          if (!geometryStyle.fill.color) {
            geometryStyle.fill.color = defaultGeometryStyle.fill.color;
          }
          if (geometryStyle.fill.opacity === undefined || geometryStyle.fill.opacity === null) {
            geometryStyle.fill.opacity = defaultGeometryStyle.fill.opacity;
          }
        }
        if (!geometryStyle.border) {
          geometryStyle.border = defaultGeometryStyle.border;
        } else {
          if (!geometryStyle.border.color) {
            geometryStyle.border.color = defaultGeometryStyle.border.color;
          }
          if (geometryStyle.border.width === undefined || geometryStyle.border.width === null) {
            geometryStyle.border.width = defaultGeometryStyle.border.width;
          }
        }
        let fillColor = asArray_ol_Color(geometryStyle.fill.color),
          borderColor = asArray_ol_Color(geometryStyle.border.color);
        fillColor[3] = geometryStyle.fill.opacity;
        style = new Style({
          fill: new Fill({
            color: fillColor
          }),
          stroke: new Stroke({
            color: borderColor,
            width: geometryStyle.border.width
          }),
          image: new CircleStyle({
            // radius主要是在图形是个点时，有个点状图形
            radius: geometryStyle.border.width,
            stroke: new Stroke({
              color: fillColor
            }),
            fill: new Fill({
              color: fillColor
            })
          })
        });
      }
    }
    const isEPSG4326 = map && this.haveEPSG4326ProjectionInMapLayers(map);
    if (isEPSG4326) {
      // geometry['flatCoordinates'] = fromLonLat_ol_proj(geometry.getFlatCoordinates());
      geometry = this.convertGeometryCoordinatesFromLonLat(geometry);
    }

    const feature = new Feature({
      geometry: geometry,
    });

    feature.setStyle(style);

    const vectorLayer = new VectorLayer({
      source: new VectorSource({
        features: [feature]
      })
    });
    vectorLayer.set('id', layerId);
    vectorLayer.setZIndex(HIGHLIGHT_AREA_LAYER_INDEX);
    return vectorLayer;
  };

  /**  创建显示多个气泡框元素所在的区域的图层
   *
   *
   * @params {string} layerId
   * @params {NgSipsdMapPopUp[]} popupList
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.createPopUpListAreaLayer = function (layerId, popupList, map) {
    const features = [],
      isEPSG4326 = map && this.haveEPSG4326ProjectionInMapLayers(map);
    for (const popup of popupList) {
      let geometry = this.convertWkt2Geometry(popup.geometry),
        style;
      const defaultGeometryStyle = {
        fill: {
          color: '#3199ff',
          opacity: 0.5
        },
        border: {
          color: '#3199ff',
          width: 2
        }
      };
      let geometryStyle = popup.geometryStyle;
      if (!geometryStyle) {
        geometryStyle = defaultGeometryStyle;
      } else {
        if (geometryStyle.customStyle) {
          style = geometryStyle.customStyle;
        } else {
          if (!geometryStyle.fill) {
            geometryStyle.fill = defaultGeometryStyle.fill;
          } else {
            if (!geometryStyle.fill.color) {
              geometryStyle.fill.color = defaultGeometryStyle.fill.color;
            }
            if (geometryStyle.fill.opacity === undefined || geometryStyle.fill.opacity === null) {
              geometryStyle.fill.opacity = defaultGeometryStyle.fill.opacity;
            }
          }
          if (!geometryStyle.border) {
            geometryStyle.border = defaultGeometryStyle.border;
          } else {
            if (!geometryStyle.border.color) {
              geometryStyle.border.color = defaultGeometryStyle.border.color;
            }
            if (geometryStyle.border.width === undefined || geometryStyle.border.width === null) {
              geometryStyle.border.width = defaultGeometryStyle.border.width;
            }
          }
          let fillColor = asArray_ol_Color(geometryStyle.fill.color),
            borderColor = asArray_ol_Color(geometryStyle.border.color);
          fillColor[3] = geometryStyle.fill.opacity;
          style = new Style({
            fill: new Fill({
              color: fillColor
            }),
            stroke: new Stroke({
              color: borderColor,
              width: geometryStyle.border.width
            }),
            image: new CircleStyle({
              // radius主要是在图形是个点时，有个点状图形
              radius: geometryStyle.border.width,
              stroke: new Stroke({
                color: fillColor
              }),
              fill: new Fill({
                color: fillColor
              })
            })
          });
        }
      }
      if (isEPSG4326) {
        // geometry['flatCoordinates'] = fromLonLat_ol_proj(geometry.getFlatCoordinates());
        geometry = this.convertGeometryCoordinatesFromLonLat(geometry);
      }

      const feature = new Feature({
        geometry: geometry,
      });
      feature.setStyle(style);
      features.push(feature);
    }

    const vectorLayer = new VectorLayer({
      source: new VectorSource({
        features: features
      })
    });
    vectorLayer.set('id', layerId);
    vectorLayer.setZIndex(HIGHLIGHT_AREA_LAYER_INDEX);
    return vectorLayer;
  };
  /** 在地图上画图形
   *
   *
   * @params {*} map
   * @params {V_SIPSD_MAP_OPENLAYER_DRAW_TYPE} type
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.drawGraphics = function (map, type, ) {
    return new Promise((resolve, reject) => {

      let geometryFuncton,
        freehand = false;
      if (type === V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.CIRCLE) {
        freehand = true;
      } else if (type === V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.RECTANGLE) {
        geometryFuncton = createBox_ol_interaction_Draw();
        freehand = true;
      }

      const drawGraphicLayer = this.getDrawVector(map),
        draw = new Draw({
          source: drawGraphicLayer.getSource(),
          type: type.toString(),
          geometryFunction: geometryFuncton,
          freehand: freehand
        });

      draw.on('drawend', event => {
        // 慢一个节拍,避免快速点击两次后，
        // 触发刷机放大地图的事件
        setTimeout(() => {
          map.removeInteraction(draw);
        });

        resolve({
          map: map,
          geometry: event.feature.getGeometry().clone()
        });
      });

      map.addInteraction(draw);
    });
  };

  /** 格式化面积
   *
   *
   * @params {*} polygon
   * @returnses {string}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.formatArea = function (polygon) {
    const area = getArea_ol_sphere(polygon);

    let output;
    if (area > 10000) {
      output = Math.round(area / 1000000 * 100) / 100 + ' ' + 'k㎡';
    } else {
      output = Math.round(area * 100) / 100 + ' ' + '㎡';
    }
    return output;
  };

  /** 格式化长度
   *
   *
   * @params {*} line
   * @returnses {string}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.formatLength = function (line) {
    const length = getLength_ol_sphere(line);
    let output;
    if (length > 1000) {
      output = Math.round(length / 1000 * 100) / 100 + ' ' + 'km';
    } else {
      output = Math.round(length * 100) / 100 + ' ' + 'm';
    }
    return output;
  };

  /** 获取基础图层
   *
   *
   * @params {*} map
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getBaseLayer = function (map) {
    const layersArrs = map.getLayers().getArray();
    let baseLayer;
    for (const layerInstance of layersArrs) {
      const zIndex = layerInstance.getZIndex();
      if (0 === zIndex) {
        baseLayer = layerInstance;
      }
    }
    return baseLayer;
  };

  /** 获取点击点的坐标
   *
   *
   * @params {*} map
   * @params {Renderer2} renderer
   * @params {string} mapId
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getClickPoint = function (map, renderer, mapId) {
    return new Promise((resolve, reject) => {
      const elementRef = jQuery('#' + mapId)[0];
      renderer.setStyle(elementRef, 'cursor', 'help');
      map.once('click', clickEvent => {
        renderer.setStyle(elementRef, 'cursor', '');
        const isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
        const geometry = this.convertCoordinates2Geometry(
          V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POINT,
          isEPSG4326 ? toLonLat_ol_proj(clickEvent.coordinate) : clickEvent.coordinate
        );
        const pointWkt = this.convertGeometry2Wkt(geometry);
        resolve({
          map: map,
          value: pointWkt
        });
      });
    });
  };

  /** 获取绘画的矢量图层
   *
   *
   * @params {*} map
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getDrawVector = function (map, ) {
    const vector = this.getLayer(map, 'draw-vector');
    if (!vector) {
      return this.addVectorLayer(map, 'draw-vector');
    }
    return vector;
  };

  /** 获取四至范围
   *
   *
   * @params {*} map
   * @returnses {Object}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getExtent = function (map) {
    return new Promise(resolve => {
      let intervalIndex = setInterval(() => {
        const view = map.getView();
        let extents;
        try {
          extents = view.calculateExtent();
        } catch (e) {} finally {
          if (extents) {
            clearInterval(intervalIndex);
            intervalIndex = null;

            const isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
            let xmin, ymin, xmax, ymax;
            if (isEPSG4326) {
              xmin = toLonLat_ol_proj([extents[0]])[0];
              ymin = toLonLat_ol_proj([extents[1]])[0];
              xmax = toLonLat_ol_proj([extents[2]])[0];
              ymax = toLonLat_ol_proj([extents[3]])[0];
            } else {
              xmin = extents[0];
              ymin = extents[1];
              xmax = extents[2];
              ymax = extents[3];
            }
            const spatialReference = view.get('spatialReference');
            resolve({
              map: map,
              value: {
                xmin,
                ymin,
                xmax,
                ymax,
                spatialReference
              }
            });
          }
        }
      });
      // 3s后还无法获取范围值，则不再获取
      setTimeout(() => {
        if (intervalIndex) {
          clearInterval(intervalIndex);
          resolve(null);
        }
      }, 3000);
    });
  };
  /** 获取geometry几何的中心点
   *
   *
   * @params {*} geometry
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getGeometryCenter = function (geometry) {
    return getCenter_ol_extent(geometry.getExtent());
  };

  /** 根据图层id获取图层
   *
   *
   * @params {*} map
   * @params {string} layerId
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getLayer = function (map, layerId) {
    const layers = map.getLayers(),
      length = layers.getLength();
    for (let i = 0; i < length; i++) {
      const layerInstance = layers.item(i);
      if (layerInstance.get('id') === layerId) {
        return layerInstance;
      }
    }
    return null;
  };
  /** 获取当前比例尺层级
   *
   *
   * @params {*} map 地图示例
   * @returnses {number}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getLevel = function (map) {
    return map.getView().getZoom()
  };
  /** 获取气泡定位的图标设置
   *
   *
   * @params {NgSipsdMapPopUp[]} popUpList
   * @params {boolean} nullIndex 图标是否有索引
   * @returnses {olFeature[]}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.getPopUpPositionIcons = function (popUpList, nullIndex, map) {
    const iconFeatures = [],
      isEPSG4326 = map && this.haveEPSG4326ProjectionInMapLayers(map);
    for (let i = popUpList.length - 1; i > -1; i--) {
      const popGeometry = this.convertWkt2Geometry(popUpList[i].geometry),
        popGeometryCenter = isEPSG4326 ? fromLonLat_ol_proj(this.getGeometryCenter(popGeometry)) : this.getGeometryCenter(popGeometry),
        iconFeature = new Feature({
          geometry: new Point(popGeometryCenter),
        });

      iconFeature.setId(i);
      const iconStyle = new Style({
        image: new Icon(({
          anchor: [0.49, 1],
          anchorXUnits: 'fraction',
          anchorYUnits: 'fraction',
          // src: `assets/libs/sipsd/sipsdMap/poi/icon_dibiao_${nullIndex ? 'null' : i + 1}.png`
          src: nullIndex ? icon_dibiao_null : icon_dibiao_arrs[i]
        }))
      });
      iconFeature.setStyle(iconStyle);
      iconFeatures.push(iconFeature);
    }
    return iconFeatures;

  };

  /** 地图图层中是否有图层的projection投影
   * 为4326的，主要有一个，就返回true
   *
   * @params {*} map
   * @returnses {boolean}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.haveEPSG4326ProjectionInMapLayers = function (map) {
    const layers = map.getLayers();
    for (const item of layers.getArray()) {
      const projection = item.getSource().getProjection();
      if (projection) {
        const projectionCode = projection.getCode();
        if ('EPSG:4326' === projectionCode) {
          return true;
        }
      }
    }
    return false;

  };

  /** 空间参考系的wkt值中是否含有espg4326
   *
   *
   * @params {string} spatialReferenceWKT
   * @returnses {boolean}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.haveEPSG4326ProjectionInSpatialReference = function (spatialReferenceWKT) {
    const startIndex = spatialReferenceWKT.toUpperCase().indexOf('EPSG');
    const espgStrTemp = spatialReferenceWKT.slice(startIndex - 1);
    const endIndex = espgStrTemp.indexOf(']');
    const espgStr = espgStrTemp.slice(0, endIndex);
    const espgStrCode = espgStr.split(',')[1];
    return '4326' === espgStrCode || '4490' === espgStrCode;

  };

  /** 高亮显示气泡框元素对应的区域
   *
   *
   * @params {*} map
   * @params {NgSipsdMapPopUp} popUp
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.highlightPopUpArea = function (map, popUp) {
    const popUpItemHighlightLayerId = 'popUpItemHighlightLayer';
    this.removeMapLayer(map, popUpItemHighlightLayerId);
    const areaLayer = this.createPopUpItemAreaLayer(popUpItemHighlightLayerId, popUp, map);
    map.addLayer(areaLayer);
    return {
      map: map,
      layer: areaLayer
    };

  };

  /** 高亮显示多个气泡框元素对应的区域
   *
   *
   * @params {*} map
   * @params {NgSipsdMapPopUp[]} popUpList
   * @returnses
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.highlightPopUpListArea = function (map, popUpList) {
    const popUpItemHighlightLayerId = 'popUpItemHighlightLayer';
    this.removeMapLayer(map, popUpItemHighlightLayerId);
    const areaLayer = this.createPopUpListAreaLayer(popUpItemHighlightLayerId, popUpList, map);
    map.addLayer(areaLayer);
    return {
      map: map,
    };

  };

  // 初始化图层的index层级
  // 基础图层，index固定为0
  // 其他图层index至少为1
  VSipsdMapOpenlayerService.prototype.initLayerIndex = function (layerIndex, isBaseMap) {
    let index;
    if (isBaseMap) {
      index = 0;
    } else {
      if (!layerIndex) {
        index = 1;
      } else {
        index = layerIndex;
      }
    }
    return index;
  };

  VSipsdMapOpenlayerService.prototype.initStyle = function (graphicStyle) {
    const defaultStyle = {
      fill: {
        color: '#3199ff',
        opacity: 0.5
      },
      border: {
        color: '#3199ff',
        width: 2
      },
    };

    if (!graphicStyle) {
      graphicStyle = defaultStyle;
    }

    if (graphicStyle.customStyle) {
      return graphicStyle.customStyle;
    }

    let style;
    if (graphicStyle.fill) {
      if (!graphicStyle.fill.color) {
        graphicStyle.fill.color = defaultStyle.fill.color;
      }
      if (graphicStyle.fill.opacity === undefined || graphicStyle.fill.opacity === null) {
        graphicStyle.fill.opacity = defaultStyle.fill.opacity;
      }
    } else {
      graphicStyle.fill = defaultStyle.fill;
    }

    if (graphicStyle.border) {
      if (!graphicStyle.border.color) {
        graphicStyle.border.color = defaultStyle.border.color;
      }
      if (graphicStyle.border.width === undefined || graphicStyle.border.width === null) {
        graphicStyle.border.width = defaultStyle.border.width;
      }
    } else {
      graphicStyle.border = defaultStyle.border;
    }

    const fillColor = asArray_ol_Color(graphicStyle.fill.color);
    fillColor[3] = graphicStyle.fill.opacity;
    style = new Style({
      fill: new Fill({
        color: fillColor
      }),
      stroke: new Stroke({
        color: graphicStyle.border.color,
        width: graphicStyle.border.width
      }),
      image: new CircleStyle({
        radius: graphicStyle.border.width,
        stroke: new Stroke({
          color: graphicStyle.border.color
        }),
        fill: new Fill({
          color: fillColor
        })
      })
    });

    if (graphicStyle.text) {
      const textStyle = new Text();
      if (graphicStyle.text.text) {
        textStyle.setText(graphicStyle.text.text);
      }
      if (graphicStyle.text.font) {
        textStyle.setFont(graphicStyle.text.font);
      }
      style.setText(textStyle);
    }

    if (graphicStyle.icon) {
      if (graphicStyle.icon.imgSrc) {
        const iconOption = {
          src: graphicStyle.icon.imgSrc,
        };
        if (graphicStyle.icon.iconSize) {
          Reflect.set(iconOption, 'size', graphicStyle.icon.iconSize);
        }
        if (graphicStyle.icon.imgSize) {
          Reflect.set(iconOption, 'imgSize', graphicStyle.icon.imgSize);
        }
        if (graphicStyle.icon.crossOrigin) {
          Reflect.set(iconOption, 'crossOrigin', graphicStyle.icon.crossOrigin);
        }
        const iconStyle = new Icon(iconOption);
        style.setImage(iconStyle);
      }
    }
    return style;
  };

  /** 加载arcgis的动态地图
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriDynamicLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadArcgisDynamicLayer = function (map, layerInfo, index, isBaseMap) {
    const view = map.getView();
    if (view.getCenter() === null) {
      let layerMetaInfoUrl = _.cloneDeep(layerInfo.url);
      if (!this.checkLayerUrlValid(layerMetaInfoUrl)) {
        return new Promise((resolve, reject) => {
          resolve({
            map: map,
            layer: undefined
          });
        });
      }
      const paramsTemp = layerInfo.params;
      const paramsTempKeys = paramsTemp ? Object.keys(paramsTemp) : [];
      let isFirstParam = true;
      if (paramsTempKeys.length > 0) {
        for (const p of paramsTempKeys) {
          const paramValue = paramsTemp[p];
          if (isFirstParam) {
            layerMetaInfoUrl = `${layerMetaInfoUrl}?${p}=${encodeURI(paramValue)}`;
            isFirstParam = false;
          } else {
            layerMetaInfoUrl = `${layerMetaInfoUrl}&${p}=${encodeURI(paramValue)}`;
          }
        }
        layerMetaInfoUrl += '&f=json';
      } else {
        layerMetaInfoUrl += '?f=json';
      }
      return this.dataRequest.getJsonDataMethodByPromise(layerMetaInfoUrl).then(callback => {
        let {
          status,
          responseData
        } = callback;
        if (200 === status && this.checkLayerInfoValid(responseData)) {
          const layerMetaInfo = responseData;
          if (!layerMetaInfo.spatialReference) {
            this.layerService.showErrorInfo(`空间参考信息为“${layerMetaInfo.spatialReference}”,无法加载图层`);
            return {
              map: map,
              layer: undefined
            };
          }
          if (!layerMetaInfo.fullExtent) {
            this.layerService.showErrorInfo(`地图范围为“${layerMetaInfo.fullExtent}”,无法加载图层`);
            return {
              map: map,
              layer: undefined
            };
          }

          const extents = [
            layerMetaInfo.fullExtent.xmin,
            layerMetaInfo.fullExtent.ymin,
            layerMetaInfo.fullExtent.xmax,
            layerMetaInfo.fullExtent.ymax
          ];
          view.fit(extents);
          const spatialReferenceWKT = view.get('spatialReference');
          if (!spatialReferenceWKT) {
            view.set('spatialReference', layerMetaInfo.spatialReference.wkt);
          }
          return this.loadArcgisDynamicLayer(map, layerInfo, index);
        } else {
          return {
            map: map,
            layer: undefined
          };
        }
      });
    } else {
      return new Promise((resolve, reject) => {
        const layerUrl = layerInfo.url,
          params = new Object();
        if (!this.checkLayerUrlValid(layerUrl)) {
          resolve({
            map: map,
            layer: undefined
          });
          return;
        }
        if (!_.includes(layerInfo.vls, '*')) {
          params['LAYERS'] = 'show:'.concat(_.join(layerInfo.vls));
        }

        const spatialReferenceWKT = view.get('spatialReference');
        const projectionCode = this.haveEPSG4326ProjectionInSpatialReference(spatialReferenceWKT) ? 'EPSG:4326' : 'EPSG:3857';
        const dynamicLayer = new ImageLayer({
          source: new ImageArcGISRest({
            crossOrigin: 'anonymous',
            url: layerUrl,
            params: params,
            projection: null,
            imageLoadFunction: function (image, src) {
              const projection = {
                WKT: map.getView().get('spatialReference')
              };
              src = src.replace(
                /(BBOXSR=)(\w)+(?=&)/g,
                '$1' + JSON.stringify(projection)
              ).replace(
                /(IMAGESR=)(\w)+(?=&)/g,
                '$1' + JSON.stringify(projection)
              );
              const _image = image.getImage();
              _image.src = src;
              const layerInfoParams = layerInfo.params;
              const layerInfoParamKeys = layerInfoParams ? Object.keys(layerInfoParams) : [];
              for (const item of layerInfoParamKeys) {
                _image.src = `${_image.src}&${item}=${encodeURI(layerInfoParams[item])}`;
              }
            }
          })
        });

        index = this.initLayerIndex(index, isBaseMap);
        dynamicLayer.setZIndex(index);

        dynamicLayer.set('id', layerInfo.id);
        map.addLayer(dynamicLayer);
        resolve({
          map: map,
          layer: dynamicLayer
        });
      });
    }

  };

  /** 加载默认图层
   *
   *
   * @params {*} map
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadDefaultLayers = function (map, defaultlayer) {
    return this.loadMapLayer(map, defaultlayer, defaultlayer.index, true);
  };

  /** 加载特性图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriFeatureLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadFeatureLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('feature layer加载未实现！');
      resolve({
        map: map,
        layer: undefined
      });
    });
  };

  /** 加载返回数据是GeoJson格式的数据的图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapGeoJsonLayerInfo} layerInfo
   * @params {number} [index]
   * @params {number} [isFirstLoad]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadGeoJsonLayer = function (map, layerInfo, index, isBaseMap, isFirstLoad) {
    if (undefined === isFirstLoad || null === isFirstLoad) {
      isFirstLoad = true;
    }
    const oldLayer = this.getLayer(map, layerInfo.id);
    if (isFirstLoad === false && !oldLayer) {
      return;
    }
    if (oldLayer) {
      this.removeMapLayer(map, layerInfo.id);
    }
    let layerUrl = layerInfo.url,
      dynamicExtent = false;
    const params = layerInfo.params;
    if (params) {
      if (params['bbox'] !== undefined && params['bbox'] !== null) {
        dynamicExtent = true;
        const isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map),
          extents = map.getView().calculateExtent();
        let xmin, ymin, xmax, ymax;
        if (isEPSG4326) {
          xmin = toLonLat_ol_proj(extents[0]);
          ymin = toLonLat_ol_proj(extents[1]);
          xmax = toLonLat_ol_proj(extents[2]);
          ymax = toLonLat_ol_proj(extents[3]);
        } else {
          xmin = extents[0];
          ymin = extents[1];
          xmax = extents[2];
          ymax = extents[3];
        }
        params['bbox'] = `${xmin},${ymin},${xmax},${ymax}`;
      }
      const paramsKeys = Object.keys(params);
      let isFirstParam = true;
      if (paramsKeys.length > 0) {
        for (const p of paramsKeys) {
          const paramValue = params[p];
          if (isFirstParam) {
            layerUrl = `${layerUrl}?${p}=${encodeURI(paramValue)}`;
            isFirstParam = false;
          } else {
            layerUrl = `${layerUrl}&${p}=${encodeURI(paramValue)}`;
          }
        }
      }
    }

    return this.dataRequest.getJsonDataMethodByPromise(layerUrl, undefined, false).then(callback => {
      let {
        status,
        responseData
      } = callback;
      if (200 === status) {
        const geoJsonData = responseData;
        const geoJsonFeatures = this.convertGeojson2Features(geoJsonData);
        const defaultStyle = this.initStyle(geoJsonData.style || layerInfo.style);
        const vectorSource = new VectorSource({
          features: geoJsonFeatures
        });
        const vectorLayer = new VectorLayer({
          source: vectorSource,
          style: (feature) => {
            const featureStyle = feature.getProperties().style;
            if (featureStyle) {
              let graphicStyle = featureStyle;

              const defaultStyle = {
                fill: {
                  color: '#3199ff',
                  opacity: 0.5
                },
                border: {
                  color: '#3199ff',
                  width: 2
                },
              };

              if (!graphicStyle) {
                graphicStyle = defaultStyle;
              }

              if (graphicStyle.customStyle) {
                return graphicStyle.customStyle;
              }

              if (graphicStyle.fill) {
                if (!graphicStyle.fill.color) {
                  graphicStyle.fill.color = defaultStyle.fill.color;
                }
                if (graphicStyle.fill.opacity === undefined || graphicStyle.fill.opacity === null) {
                  graphicStyle.fill.opacity = defaultStyle.fill.opacity;
                }
              } else {
                graphicStyle.fill = defaultStyle.fill;
              }

              if (graphicStyle.border) {
                if (!graphicStyle.border.color) {
                  graphicStyle.border.color = defaultStyle.border.color;
                }
                if (graphicStyle.border.width === undefined || graphicStyle.border.width === null) {
                  graphicStyle.border.width = defaultStyle.border.width;
                }
              } else {
                graphicStyle.border = defaultStyle.border;
              }

              const fillColor = asArray_ol_Color(graphicStyle.fill.color);
              fillColor[3] = graphicStyle.fill.opacity;
              const style = [new Style({
                fill: new Fill({
                  color: fillColor
                }),
                stroke: new Stroke({
                  color: graphicStyle.border.color,
                  width: graphicStyle.border.width
                }),
                image: new CircleStyle({
                  radius: graphicStyle.border.width,
                  stroke: new Stroke({
                    color: graphicStyle.border.color
                  }),
                  fill: new Fill({
                    color: fillColor
                  })
                })
              })];

              if (graphicStyle.text) {
                const textStyle = new Text();
                if (graphicStyle.text.text) {
                  textStyle.setText(graphicStyle.text.text);
                }
                if (graphicStyle.text.font) {
                  textStyle.setFont(graphicStyle.text.font);
                }
                style.setText(textStyle);
              }

              if (graphicStyle.icon) {
                style[0].setImage(null);
                const geometry = feature.getGeometry();
                switch (geometry.getType()) {
                  case 'Point':
                    {
                      if (graphicStyle.icon.imgSrc) {
                        const iconOption = {
                          src: graphicStyle.icon.imgSrc,
                          color: fillColor,
                        };
                        if (graphicStyle.icon.scale) {
                          Reflect.set(iconOption, 'scale', graphicStyle.icon.scale);
                        }
                        if (graphicStyle.icon.color) {
                          Reflect.set(iconOption, 'color', graphicStyle.icon.color);
                        }
                        if (graphicStyle.icon.crossOrigin) {
                          Reflect.set(iconOption, 'crossOrigin', graphicStyle.icon.crossOrigin);
                        }
                        style.push(new Style({
                          geometry: new Point(this.getGeometryCenter(geometry)),
                          image: new Icon(iconOption)
                        }));
                      }
                      break;
                    }
                  case 'LineString':
                    {
                      geometry.forEachSegment((start, end) => {
                        const dx = end[0] - start[0];
                        const dy = end[1] - start[1];

                        const rotation = Math.atan2(dy, dx);
                        // arrows
                        const unit = 15;
                        const dz = Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
                        let slices = Math.floor(dz / unit);
                        slices = slices > 1 ? slices : 2;
                        for (let i = 1; i < slices; i++) {
                          const iconOption = {
                            src: graphicStyle.icon.imgSrc,
                            anchor: [0.2, 0.5],
                            color: fillColor,
                            rotateWithView: true,
                            rotation: -rotation
                          };
                          if (graphicStyle.icon.scale) {
                            Reflect.set(iconOption, 'scale', graphicStyle.icon.scale);
                          }
                          if (graphicStyle.icon.color) {
                            Reflect.set(iconOption, 'color', graphicStyle.icon.color);
                          }
                          const percentage = i / slices;
                          const distanceX = dx * percentage;
                          const distanceY = dy * percentage;
                          style.push(new Style({
                            geometry: new Point([start[0] + distanceX, start[1] + distanceY]),
                            // geometry: new Point(end),
                            image: new Icon(iconOption)
                          }));
                        }
                      });
                      break;
                    }
                  default:
                    {
                      break;
                    }
                }
              }
              return style;
            } else {
              return defaultStyle;
            }
          },
        });
        index = this.initLayerIndex(index, isBaseMap);
        vectorLayer.setZIndex(index);

        vectorLayer.set('id', layerInfo.id);
        map.addLayer(vectorLayer, index);
        // 如果是需要动态获取视图view范围的，
        // 那么每次拖动，放大缩小时都需要重新获取,
        // 如果原来的图层已经被移除，那么就不用再动态获取视图view范围，
        // 获取相关数据
        if (dynamicExtent && (!!(isFirstLoad || oldLayer))) {
          map.once('moveend', ($event) => {
            this.loadGeoJsonLayer(map, layerInfo, index, isBaseMap, false);
          });
        }
        return {
          map: map,
          layer: vectorLayer
        };
      } else {
        return {
          map: map,
          layer: undefined
        };
      }

    });
  };

  /** 加载图形图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapGraphicLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadGraphicLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve, reject) => {
      let vectorLayer;
      if (layerInfo.graphicList) {
        if (!this.getLayer(map, layerInfo.id)) {
          const features = [];
          for (const item of layerInfo.graphicList) {
            const geometry = this.convertWkt2Geometry(item.geometry),
              defaultStyle = {
                fill: {
                  color: '#3199ff',
                  opacity: 0.5
                },
                border: {
                  color: '#3199ff',
                  width: 2
                },
              };
            let style;
            if (!item.style) {
              item.style = defaultStyle;
            }
            if (item.style.customStyle) {
              style = item.style.customStyle;
            } else {
              if (item.style.fill) {
                if (!item.style.fill.color) {
                  item.style.fill.color = defaultStyle.fill.color;
                }
                if (item.style.fill.opacity === undefined || item.style.fill.opacity === null) {
                  item.style.fill.opacity = defaultStyle.fill.opacity;
                }
              } else {
                item.style.fill = defaultStyle.fill;
              }
              if (item.style.border) {
                if (!item.style.border.color) {
                  item.style.border.color = defaultStyle.border.color;
                }
                if (item.style.border.width === undefined || item.style.border.width === null) {
                  item.style.border.width = defaultStyle.border.width;
                }
              } else {
                item.style.border = defaultStyle.border;
              }
              const fillColor = asArray_ol_Color(item.style.fill.color);
              fillColor[3] = item.style.fill.opacity;
              style = new Style({
                fill: new Fill({
                  color: fillColor
                }),
                stroke: new Stroke({
                  color: item.style.border.color,
                  width: item.style.border.width
                }),
                image: new CircleStyle({
                  radius: item.style.border.width,
                  stroke: new Stroke({
                    color: item.style.border.color
                  }),
                  fill: new Fill({
                    color: fillColor
                  })
                })
              });
            }
            const feature = new Feature({
              geometry: geometry,
            });
            feature.setStyle(style);
            features.push(feature);
          }
          const source = new VectorSource({
            features: features
          });
          const vectorLayer = new VectorLayer({
            source: source
          });
          vectorLayer.set('id', layerInfo.id);

          index = this.initLayerIndex(index, isBaseMap);
          vectorLayer.setZIndex(index);

          map.addLayer(vectorLayer);
        } else {
          this.layerService.showErrorInfo(`id为${layerInfo.id}的“${layerInfo.name}”图层已存在,无法重复加载!`);
        }
      } else {
        this.layerService.showErrorInfo('图层的graphicList图形列表数据缺失，无法加载该图层！');
      }

      resolve({
        map: map,
        layer: vectorLayer
      });
    });
  };

  /** 加载图层
   *
   *
   * @params {*} map
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadMapLayer = function (map, layerInfo, index, isBaseMap) {
    if (layerInfo && layerInfo.url) {
      const layerUrlList = layerInfo.url.split('?');
      if (layerUrlList.length > 1 && layerUrlList[1]) {
        const paramUrlList = layerUrlList[1].split('&');
        paramUrlList.forEach(paramUrl => {
          const paramList = paramUrl.split('=');
          if (!layerInfo.params) {
            layerInfo.params = {};
          }
          Reflect.set(layerInfo.params, paramList[0], paramList[1]);
        });
      }
      layerInfo.url = layerUrlList[0];
    }

    const layerInfoType = layerInfo.type.toLowerCase();
    if (V_SIPSD_MAP_TYPE.ESRI_TILEDLAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.TILE_SERVICES.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.ARCGIS_TILE.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.TILE.toLowerCase() === layerInfoType) {
      return this.loadTiledLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.VECTOR_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadVectorTiledLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.ESRI_DYNAMICLAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.ARCGIS_DYN.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.DYNAMIC.toLowerCase() === layerInfoType) {
      return this.loadArcgisDynamicLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.ESRI_FEATURELAYER.toLowerCase() === layerInfoType) {
      return this.loadFeatureLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.TIANDI_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadTianMapLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.ESRI_WMSLAYER.toLowerCase() === layerInfoType) {
      return this.loadWMSLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.GRAPHIC_LAYER.toLowerCase() === layerInfoType) {
      return this.loadGraphicLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.GEOJSON_LAYER.toLowerCase() === layerInfoType ||
      V_SIPSD_MAP_TYPE.GEOJSON.toLowerCase() === layerInfoType) {
      return this.loadGeoJsonLayer(map, layerInfo, index, isBaseMap);
    } else if (V_SIPSD_MAP_TYPE.VECTOR_GEOJSON_TILEDLAYER.toLowerCase() === layerInfoType) {
      return this.loadVectorGeoJsonTiledLayer(map, layerInfo, index, isBaseMap);
    } else {
      return new Promise((resolve, reject) => {
        // this.layerService.showWarnInfo(`“${layerInfo.type}”为未支持的地图类型，无法加载该图层！`);
        console.warn(`“${layerInfoType}”为未支持的地图类型，在组件中通过“ngSingleMapSelectLayersTreeChange”方法获得加载的图层列表后，请自行实现加载方法！`);
        resolve({
          map: map
        });
      });
    }
  };

  /** 加载切片图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriTiledLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadTiledLayer = function (map, layerInfo, index, isBaseMap) {
    let layerUrl = `${layerInfo.url}/tile/{z}/{y}/{x}`,
      layerMetaInfoUrl = _.cloneDeep(layerInfo.url);
    if (!this.checkLayerUrlValid(layerMetaInfoUrl)) {
      return new Promise((resolve, reject) => {
        resolve({
          map: map
        });
      });
    }
    const params = layerInfo.params;
    const paramsKeys = params ? Object.keys(params) : [];
    let isFirstParam = true;
    if (paramsKeys.length > 0) {
      for (const p of paramsKeys) {
        const paramValue = params[p];
        if (isFirstParam) {
          layerUrl = `${layerUrl}?${p}=${encodeURI(paramValue)}`;
          layerMetaInfoUrl = `${layerMetaInfoUrl}?${p}=${encodeURI(paramValue)}`;
          isFirstParam = false;
        } else {
          layerUrl = `${layerUrl}&${p}=${encodeURI(paramValue)}`;
          layerMetaInfoUrl = `${layerMetaInfoUrl}&${p}=${encodeURI(paramValue)}`;
        }
      }
      layerMetaInfoUrl += '&f=json';
    } else {
      layerMetaInfoUrl += '?f=json';
    }
    return this.dataRequest.getJsonDataMethodByPromise(layerMetaInfoUrl).then(callback => {
      let {
        status,
        responseData
      } = callback;
      if (200 === status && this.checkLayerInfoValid(responseData)) {
        const layerMetaInfo = responseData;
        if (!layerMetaInfo.spatialReference) {
          this.layerService.showErrorInfo(`空间参考信息为“${layerMetaInfo.spatialReference}”,无法加载图层`);
          return {
            map: map,
            layer: undefined
          };
        }
        if (!layerMetaInfo.fullExtent) {
          this.layerService.showErrorInfo(`地图范围为“${layerMetaInfo.fullExtent}”,无法加载图层`);
          return {
            map: map,
            layer: undefined
          };
        }
        if (!layerMetaInfo.tileInfo) {
          this.layerService.showErrorInfo(`切片信息为“${layerMetaInfo.tileInfo}”,无法加载图层`);
          return {
            map: map,
            layer: undefined
          };
        }
        const spatialReferenceWKT = layerMetaInfo.spatialReference.wkt;
        const projection = this.haveEPSG4326ProjectionInSpatialReference(spatialReferenceWKT) ? 'EPSG:4326' : 'EPSG:3857';
        const tileLayer = new TileLayer({
          source: new XYZ({
            crossOrigin: 'anonymous',
            url: layerUrl,
            projection: projection,
            tileGrid: new TileGrid({
              extent: [
                layerMetaInfo.fullExtent.xmin,
                layerMetaInfo.fullExtent.ymin,
                layerMetaInfo.fullExtent.xmax,
                layerMetaInfo.fullExtent.ymax
              ],
              origin: [
                layerMetaInfo.tileInfo.origin.x,
                layerMetaInfo.tileInfo.origin.y
              ],
              resolutions: _.map(layerMetaInfo.tileInfo.lods, 'resolution'),
              tileSize: [
                layerMetaInfo.tileInfo.cols,
                layerMetaInfo.tileInfo.rows
              ]
            })
          })
        });

        index = this.initLayerIndex(index, isBaseMap);
        tileLayer.setZIndex(index);

        tileLayer.set('id', layerInfo.id);
        tileLayer.set('spatialReference', spatialReferenceWKT);
        map.addLayer(tileLayer);

        return {
          map: map,
          layer: tileLayer
        };
      } else {
        return {
          map: map,
          layer: undefined
        };
      }
    });
  };

  /** 加载geoJson类型的矢量切片图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapVectorGeoJsonTiledLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<Object>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadVectorGeoJsonTiledLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve) => {
      let layerUrl = layerInfo.url;
      const params = layerInfo.params;
      if (params) {
        const paramsKeys = Object.keys(params);
        let isFirstParam = true;
        for (const p of paramsKeys) {
          if (isFirstParam) {
            layerUrl = `${layerUrl}?${p}=${encodeURI(params[p])}`;
            isFirstParam = false;
          } else {
            layerUrl = `${layerUrl}&${p}=${encodeURI(params[p])}`;
          }
        }
      }
      const defaultStyle = {
        fill: {
          color: '#3199ff',
          opacity: 0.5
        },
        border: {
          color: '#3199ff',
          width: 2
        },
      };
      let style;
      if (!layerInfo.style) {
        layerInfo.style = defaultStyle;
      }
      if (layerInfo.style.customStyle) {
        style = layerInfo.style.customStyle;
      } else {
        if (layerInfo.style.fill) {
          if (!layerInfo.style.fill.color) {
            layerInfo.style.fill.color = defaultStyle.fill.color;
          }
          if (layerInfo.style.fill.opacity === undefined || layerInfo.style.fill.opacity === null) {
            layerInfo.style.fill.opacity = defaultStyle.fill.opacity;
          }
        } else {
          layerInfo.style.fill = defaultStyle.fill;
        }
        if (layerInfo.style.border) {
          if (!layerInfo.style.border.color) {
            layerInfo.style.border.color = defaultStyle.border.color;
          }
          if (layerInfo.style.border.width === undefined || layerInfo.style.border.width === null) {
            layerInfo.style.border.width = defaultStyle.border.width;
          }
        } else {
          layerInfo.style.border = defaultStyle.border;
        }
        const fillColor = asArray_ol_Color(layerInfo.style.fill.color);
        fillColor[3] = layerInfo.style.fill.opacity;
        style = new Style({
          fill: new Fill({
            color: fillColor
          }),
          stroke: new Stroke({
            color: layerInfo.style.border.color,
            width: layerInfo.style.border.width
          }),
          image: new CircleStyle({
            radius: layerInfo.style.border.width,
            stroke: new Stroke({
              color: layerInfo.style.border.color
            }),
            fill: new Fill({
              color: fillColor
            })
          })
        });
      }

      const vertorTiledLayer = new VectorTileLayer({
        declutter: true,
        source: new VectorTile({
          // format: new MVT(),
          format: new GeoJSON,
          // projection: "EPSG:4326",
          url: layerUrl
        }),
        style: style
      });

      index = this.initLayerIndex(index, isBaseMap);
      vertorTiledLayer.setZIndex(index);

      vertorTiledLayer.set('id', layerInfo.id);
      map.addLayer(vertorTiledLayer);

      resolve({
        map: map,
        layer: vertorTiledLayer
      });
    });
  };

  /** 加载矢量切片图层
   *
   *
   * @params {*} map 地图实例
   * @params {NgSipsdMapVectorTiledLayerInfo} layerInfo 图层信息
   * @params {number} [index] 层级
   * @returnses {Promise<Object>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadVectorTiledLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve) => {
      let layerUrl = layerInfo.url;
      const params = layerInfo.params;
      if (params) {
        const paramsKeys = Object.keys(params);
        let isFirstParam = true;
        for (const p of paramsKeys) {
          if (isFirstParam) {
            layerUrl = `${layerUrl}?${p}=${encodeURI(params[p])}`;
            isFirstParam = false;
          } else {
            layerUrl = `${layerUrl}&${p}=${encodeURI(params[p])}`;
          }
        }
      }

      const defaultStyle = {
        fill: {
          color: '#3199ff',
          opacity: 0.5
        },
        border: {
          color: '#3199ff',
          width: 2
        },
      };
      let style;
      if (!layerInfo.style) {
        layerInfo.style = defaultStyle;
      }
      if (layerInfo.style.customStyle) {
        style = layerInfo.style.customStyle;
      } else {
        if (layerInfo.style.fill) {
          if (!layerInfo.style.fill.color) {
            layerInfo.style.fill.color = defaultStyle.fill.color;
          }
          if (layerInfo.style.fill.opacity === undefined || layerInfo.style.fill.opacity === null) {
            layerInfo.style.fill.opacity = defaultStyle.fill.opacity;
          }
        } else {
          layerInfo.style.fill = defaultStyle.fill;
        }
        if (layerInfo.style.border) {
          if (!layerInfo.style.border.color) {
            layerInfo.style.border.color = defaultStyle.border.color;
          }
          if (layerInfo.style.border.width === undefined || layerInfo.style.border.width === null) {
            layerInfo.style.border.width = defaultStyle.border.width;
          }
        } else {
          layerInfo.style.border = defaultStyle.border;
        }
        const fillColor = asArray_ol_Color(layerInfo.style.fill.color);
        fillColor[3] = layerInfo.style.fill.opacity;
        style = new Style({
          fill: new Fill({
            color: fillColor
          }),
          stroke: new Stroke({
            color: layerInfo.style.border.color,
            width: layerInfo.style.border.width
          }),
          image: new CircleStyle({
            radius: layerInfo.style.border.width,
            stroke: new Stroke({
              color: layerInfo.style.border.color
            }),
            fill: new Fill({
              color: fillColor
            })
          })
        });
      }
      const vertorTiledLayer = new VectorTileLayer({
        declutter: true,
        source: new VectorTile({
          format: new MVT(),
          // projection: "EPSG:4326",
          url: layerUrl
        }),
        style: style
      });

      index = this.initLayerIndex(index, isBaseMap);
      vertorTiledLayer.setZIndex(index);

      vertorTiledLayer.set('id', layerInfo.id);
      map.addLayer(vertorTiledLayer);

      // const key = 'pk.eyJ1IjoiYTU5Nzg2MzM5NSIsImEiOiJjampxcXMydGswZTZyM3Zwbjk5NmFpZzhqIn0.gL0H-xYr8A-KvF8LXJGOiA';
      // let layer3 = new VectorTileLayer({
      //   declutter: true,
      //   source: new VectorTile({
      //     attributions: '© <a href="https://www.mapbox.com/map-feedback/">Mapbox</a> ' +
      //       '© <a href="https://www.openstreetmap.org/copyright">' +
      //       'OpenStreetMap contributors</a>',
      //     format: new MVT(),
      //     // projection: "EPSG:4326",
      //     url: "https://{a-d}.tiles.mapbox.com/v4/mapbox.mapbox-streets-v6/{z}/{x}/{y}.vector.pbf?access_token=" + key,

      //   }),
      //   style: new Style({
      //     fill: new Fill({
      //       color: 'rgba(255, 255, 255, 0.2)'
      //     }),
      //     stroke: new Stroke({
      //       color: 'rgba(0, 0, 0, 0.5)',
      //       lineDash: [10, 10],
      //       width: 2
      //     }),
      //     image: new CircleStyle({
      //       radius: 5,
      //       stroke: new Stroke({
      //         color: 'rgba(0, 0, 0, 0.7)'
      //       }),
      //       fill: new Fill({
      //         color: 'rgba(255, 255, 255, 0.2)'
      //       })
      //     })
      //   })
      // });

      // map.addLayer(layer3);
      resolve({
        map: map,
        layer: vertorTiledLayer
      });
    });
  };

  /** 加载天地图
   *
   *
   * @params {*} map
   * @params {*} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadTianMapLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve, reject) => {
      const tiledLayer = new TileLayer({
        source: new TianMap({
          // mapType: 'satLabel',
          projection: get_ol_proj('EPSG:4326')
        })
      });

      const labelLayer = new TileLayer({
        source: new TianMap({
          mapType: 'label',
          projection: get_ol_proj('EPSG:4326')
        })
      });

      index = this.initLayerIndex(index, isBaseMap);
      tiledLayer.setZIndex(index);
      labelLayer.setZIndex(index);

      tiledLayer.set('id', layerInfo.id);
      tiledLayer.set('spatialReference', 'EPSG:4326');
      labelLayer.set('spatialReference', 'EPSG:4326');
      map.addLayer(tiledLayer);
      map.addLayer(labelLayer);
      resolve({
        map: map,
        layer: labelLayer
      });
    });
  };

  /** 加载wms类型的图层
   *
   *
   * @params {*} map
   * @params {NgSipsdMapEsriWMSLayerInfo} layerInfo
   * @params {number} [index]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.loadWMSLayer = function (map, layerInfo, index, isBaseMap) {
    return new Promise((resolve, reject) => {
      this.layerService.showWarnInfo('wms layer加载未实现！');
      resolve({
        map: map
      });
    });
  };

  /** 在地图中测量面积
   *
   *
   * @params {*} map
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.measureArea = function (map, ) {
    return this.addInteraction(map, V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYGON);

  };

  /** 在地图中测量距离
   *
   *
   * @params {*} map map对象
   * @params {*} drawTool 画图工具对象
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.measureLength = function (map, ) {
    return this.addInteraction(map, V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYLINE);

  };

  /** 清除指定的图层
   *
   *
   * @params {*} map
   * @params {string} layerId
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.removeMapLayer = function (map, layerId) {
    const layerInstance = this.getLayer(map, layerId);
    if (layerInstance) {
      map.removeLayer(layerInstance);
    } else {
      console.warn('未找到此图层！layerId:' + layerId);
    }
    return map;

  };
  /** 在地图上选择圆形区域
   *
   *
   * @params {*} map
   * @returnses {Promise<Object>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.selectCircle = function (map, ) {
    return this.drawGraphics(map, V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.CIRCLE).then(callback => {
      map = callback['map'];
      const geometry = this.convertGeometry2Wkt(this.convertCircleGeometry2PolygonGeometry(callback['geometry']));
      return {
        result: true,
        map: map,
        value: geometry
      };
    });
  };

  /** 在地图上选择矩形区域
   *
   *
   * @params {*} map
   * @returnses {Promise<Object>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.selectRectangle = function (map, ) {
    return this.drawGraphics(map, V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.RECTANGLE).then(callback => {
      map = callback['map'];
      const geometry = this.convertGeometry2Wkt(callback['geometry']);
      return {
        result: true,
        map: map,
        value: geometry
      };
    });
  };

  /** 在地图上选择多边形区域
   *
   *
   * @params {*} map
   * @returnses {Promise<Object>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.selectPolygon = function (map, ) {
    return this.drawGraphics(map, V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYGON).then(callback => {
      map = callback['map'];
      const geometry = this.convertGeometry2Wkt(callback['geometry']);
      return {
        result: true,
        map: map,
        value: geometry
      };
    });
  };

  /** 设置范围
   *
   *
   * @params {*} map map对象
   * @params {number[]} extent 范围信息[xmin,ymin,xmax,ymax]
   * @params {number} [zoomLevel] 缩放层级
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setExtent = function (map, extent, zoomLevel) {
    return this.setMapExtent(map, extent, zoomLevel);
  };

  /** 设置地图初始化范围
   *
   *
   * @params {*} map map对象
   * @params {NgSipsdMapBaseInfo} mapInfo 地图信息
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setInitExtent = function (map, mapInfo, zoomLevel) {
    const extents = [
      mapInfo.extent.xmin,
      mapInfo.extent.ymin,
      mapInfo.extent.xmax,
      mapInfo.extent.ymax,
    ];
    return this.setMapExtent(map, extents, zoomLevel);

  };

  /** 设置地图范围
   *
   *
   * @params {*} map
   * @params {number[]} extents
   * @params {number} [zoomLevel]
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setMapExtent = function (map, extents, zoomLevel) {
    return new Promise((resolve, reject) => {
      const view = map.getView(),
        isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
      let extentDatas;
      if (isEPSG4326) {
        extentDatas = [];
        for (const item of extents) {
          extentDatas.push(fromLonLat_ol_proj([item])[0]);
        }
      } else {
        extentDatas = extents;
      }
      view.fit(extentDatas);
      if (isEPSG4326) {
        // 中心的点设置要在范围设置之后才行，
        // 否则设置范围之后，还会自己计算中心点
        // 这边要自己设置中心点，主要是因为在坐标转换过程中，
        // 数据会有误差，中心点不是原来坐标的中心点了
        const centerPoint = fromLonLat_ol_proj(getCenter_ol_extent(extents));
        view.setCenter(centerPoint);
      }
      if (zoomLevel) {
        view.setZoom(zoomLevel);
      }
      resolve({
        map: map
      });
    });

  };

  /** 设置地图窗口
   *
   *
   * @params {*} map
   * @params {string} spatialReference
   * @params {Array<number>} resolutions
   * @returnses {Object}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setMapView = function (map, spatialReference, resolutions) {
    let resolutionDatas;
    const isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
    if (isEPSG4326) {
      resolutionDatas = [];
      for (const item of resolutions) {
        resolutionDatas.push(fromLonLat_ol_proj([item])[0]);
      }
    } else {
      resolutionDatas = resolutions;
    }
    const view = new View({
      // 分辨率设置
      resolutions: resolutionDatas,
      // 默认投影是3857，一般也都是3857，不用4326
      // projection: _.startsWith(spatialReference, 'EPSG') ? olProj.get('EPSG:4326') : undefined
      // zoom: 6
    });
    view.set('spatialReference', spatialReference);
    map.setView(view);
    return {
      map: map,
      view: view
    };

  };

  /** 设置鼠标平移
   *
   *
   * @params {*} map
   * @returnses {Object}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setPan = function (map, ) {
    if ((typeof map.getInteractions) === 'function') {
      const interactions = map.getInteractions().getArray(),
        typeArrs = [
          V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYLINE,
          V_SIPSD_MAP_OPENLAYER_DRAW_TYPE.POLYGON,
        ];
      for (const item of interactions) {
        if (item instanceof Draw) {
          const mode = item.mode_;
          for (const typeTemp of typeArrs) {
            if (mode === typeTemp) {
              const measureTooltip = map.getOverlayById('measureTooltip'),
                helpTooltip = map.getOverlayById('helpTooltip');
              // 移除提示信息
              map.removeOverlay(measureTooltip);
              // 移除提示信息
              map.removeOverlay(helpTooltip);
              map.removeInteraction(item);
              break;
            }
          }
        }
      }
    }
    return {
      map: map,
    };
  };

  /** 设置地图缩放程度
   *
   *
   * @params {*} map map对象
   * @params {boolean} zoomIn 是否放大，true是放大，false是缩小
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.setZoom = function (map, zoomIn) {
    return new Promise((resolve, reject) => {
      const currentZoom = map.getView().getZoom(),
        zoomLevel = zoomIn ? 1 : -1,
        zoom = currentZoom + zoomLevel;
      map.getView().setZoom(zoom);
      resolve({
        result: true,
        map: map,
      });
    });
  };

  /** 显示气泡框。每次都要新增覆盖物，否则有时候会
   * 造成气泡框的定位显示有问题
   *
   * @params {*} map
   * @params {string} popUpId
   * @params {*} popUpElement
   * @params {number[]} [coordinates]
   * @params {number[]} [offset]
   * @params {boolean} [zoom] 是否按照一定的比例尺进行缩放
   * @returnses {*}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.showMapPopUp = function (map, popUpId, popUpElement, coordinates, offset, zoom) {
    if (!offset || offset.length < 1) {
      offset = [10, -100];
    }
    if (zoom === undefined || zoom === null) {
      zoom = true;
    }

    const popUpOverlay = map.getOverlayById(popUpId);
    if (popUpOverlay) {
      // 将原来的覆盖物删除
      map.removeOverlay(popUpOverlay);
    }
    // 气泡框覆盖物对象
    const popUpElementOverlay = this.createPopUpElementOverlay(popUpId, popUpElement, coordinates, offset);
    map.addOverlay(popUpElementOverlay);

    const mapView = map.getView();
    // 根据坐标点居中显示
    mapView.centerOn(coordinates, [0, 0], [0, 0]);
    // 按照一定的比例尺进行缩放
    if (zoom) {
      mapView.setZoom(mapView.getMaxZoom() - 2);
    }

    return map;
  };

  /** 显示气泡
   *
   *
   * @params {*} map
   * @params {NgSipsdMapPopUps} popUps
   * @params {number} mapPopUpSelectedIndex
   * @params {*} popUpElement
   * @returnses {Promise<any>}
   * @memberof NgSipsdMapOpenlayerService
   */
  VSipsdMapOpenlayerService.prototype.showMapPopUps = function (map, popUps, mapPopUpSelectedIndex, popUpElement) {
    return new Promise((resolve, reject) => {
      const popUpList = popUps.popUpList,
        isEPSG4326 = this.haveEPSG4326ProjectionInMapLayers(map);
      this.clearPopUps(map);
      if (!popUpList || popUpList.length <= 0) {
        resolve({
          result: false,
          type: popUps.type,
        });
      } else {
        if ('iQuery' === popUps.type) {
          const popUp = popUpList[mapPopUpSelectedIndex];
          if (popUp) {
            const popGeometry = this.convertWkt2Geometry(popUp.geometry),
              popGeometryCenter = isEPSG4326 ? fromLonLat_ol_proj(this.getGeometryCenter(popGeometry)) : this.getGeometryCenter(popGeometry),
              hasPrev = mapPopUpSelectedIndex === 0,
              hasNext = mapPopUpSelectedIndex === popUpList.length - 1,
              popUpId = 'iQuery-popUp';

            map = this.showMapPopUp(map, popUpId, popUpElement, popGeometryCenter, [10, -100], popUps.zoom);

            // 高亮显示对应的区域
            const highlightAreaConstructorClass = {
              VectorLayer: VectorLayer,
              VectorSource: VectorSource,
              Feature: Feature,
              Color2Array: asArray_ol_Color,
            };
            this.createHighlightAreaLayer(map, [popUp], highlightAreaConstructorClass);
            popUp.hasPrev = !hasPrev;
            popUp.hasNext = !hasNext;
            popUp.index = mapPopUpSelectedIndex;
            popUp.total = popUpList.length;
            resolve({
              result: true,
              type: popUps.type,
              popUp: popUp,
              mapPopUpSelectedIndex: mapPopUpSelectedIndex,
            });
          } else {
            console.warn(`第${mapPopUpSelectedIndex + 1}个气泡框内容不存在，请检查！`);
            resolve({
              result: false,
              type: popUps.type,
              mapPopUpSelectedIndex: mapPopUpSelectedIndex,
            });
          }
        } else {
          const highlightAreaConstructorClass = {
              VectorLayer: VectorLayer,
              VectorSource: VectorSource,
              Feature: Feature,
              Color2Array: asArray_ol_Color,
            },
            iconMarkerConstructorClass = {
              VectorLayer: VectorLayer,
              VectorSource: VectorSource,
              Feature: Feature,
              Point: Point,
              Style: Style,
              Icon: Icon
            };
          this.createHighlightAreaLayer(map, popUpList, highlightAreaConstructorClass);
          const iconMarkerLayerId = ICON_MARKER_LAYER_ID;
          if ('nullIndexPopUps' === popUps.type) {
            this.createNullIndexIconMarkersLayer(map, iconMarkerLayerId, popUpList, iconMarkerConstructorClass);
          } else {
            this.createIndexIconMarkersLayer(map, iconMarkerLayerId, popUpList, iconMarkerConstructorClass);
          }

          const popUp = popUpList[mapPopUpSelectedIndex];
          const popGeometry = this.convertWkt2Geometry(popUp.geometry);
          const coordinates = isEPSG4326 ? fromLonLat_ol_proj(this.getGeometryCenter(popGeometry)) : this.getGeometryCenter(popGeometry),
            popUpId = POPUP_LAYER_ID;
          this.showMapPopUp(map, popUpId, popUpElement, coordinates, [10, -100], popUps.zoom);

          const result = popUp ? true : false;
          resolve({
            result: result,
            type: popUps.type,
            popUp: popUp,
            mapPopUpSelectedIndex: result ? mapPopUpSelectedIndex : 0,
          });
        }
      }

    });
  };

  return VSipsdMapOpenlayerService;
}());

export {
  VSipsdMapOpenlayerService
};
