const BaseToolService = (function () {

  function BaseToolService() {}

  /** 将参数加到url后面
   *
   *
   * @param {Object} params
   * @param {string} url
   * @returns {string}
   * @memberof BaseToolService
   */
  BaseToolService.prototype.clipParams2Url = function (params, url) {
    if (this.isEmptyObject(params) !== 0) {
      return url;
    }
    // 如果url是以&符号结尾，说明之前已经有参数
    let isFirstParam = !(url.includes('?') && url.includes('&')) || (url.includes('?') && url.endsWith('?'));
    if (isFirstParam) {
      url = url.endsWith('?') ? url.slice(0, url.length - 1) : url;
    }
    const paramKeys = Object.keys(params);
    for (const p of paramKeys) {
      if (isFirstParam) {
        url = `${url}?${p}=${params[p]}`;
        isFirstParam = false;
      } else {
        url = `${url}&${p}=${params[p]}`;
      }
    }
    return url;
  };

  /**将数据转为json对象 */
  BaseToolService.prototype.convert2JsonObject = function (data) {
    if ('string' === typeof data) {
      return JSON.parse(data);
    }
    return data;
  };
  /** 判断对象是否为空对象{}
   * 
   * 
   * @param {Object} obj 要判断的对象
   * @returns {number} 当对象为undefined或者null时，返回-1，当对象为空对象{}时返回1，当对象中含有属性时，返回0
   * @memberof BaseToolService
   */
  BaseToolService.prototype.isEmptyObject = function (object) {
    if (object === undefined || object === null) {
      return -1;
    }
    for (const p of Object.keys(object)) {
      if (object.hasOwnProperty(p)) {
        return 0;
      }
    }
    return 1;
  };
  /**如果属性为null或者undefined，则将其值设置为默认值 */
  BaseToolService.prototype.setPropDefaultValue = function (prop, defaultValue) {
    if (prop === undefined || prop === null) {
      prop = defaultValue;
    }
    return prop;
  };

  return BaseToolService;
}());
export {
  BaseToolService
};
