const ListenWindowService = (function() {

  function ListenWindowService(){}

  /** 监听window浏览器窗口的鼠标单击事件
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.click= function (listen, callFunc) {
    const eventName = 'click';
    this.listenEvent(eventName,listen,callFunc);
  };
  /** 监听window浏览器窗口事件
   *
   * @param {*} eventName 监听事件名
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.listenEvent= function (eventName, listen, callFunc) {
    if (true === listen) {
      window.addEventListener(eventName, callFunc);
    } else {
      window.removeEventListener(eventName, callFunc);
    }
  };
  /** 监听window浏览器窗口的鼠标进入事件
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.mouseenter= function (listen, callFunc) {
    const eventName = 'mouseenter';
    this.listenEvent(eventName,listen,callFunc);
  };
  /** 监听window浏览器窗口的鼠标离开事件
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.mouseleave= function (listen, callFunc) {
    const eventName = 'mouseleave';
    this.listenEvent(eventName,listen,callFunc);
  };
  /** 监听window浏览器窗口的鼠标移动事件
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.mousemove= function (listen, callFunc) {
    const eventName = 'mousemove';
    this.listenEvent(eventName,listen,callFunc);
  };
  /** 监听window浏览器窗口的鼠标松开事件
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.mouseup= function (listen, callFunc) {
    const eventName = 'mouseup';
    this.listenEvent(eventName,listen,callFunc);
  };
  /** 监听window浏览器窗口的大小改变
   *
   *
   * @param {*} listen 是否监听，false时会将监听的事件移除
   * @param {*} callFunc 大小改变时（listen=true）要触发的事件，或者不监听时（listen=false）要移除的函数
   */
  ListenWindowService.prototype.resize= function (listen, callFunc) {
    const eventName = 'resize';
    this.listenEvent(eventName,listen,callFunc);
  };
  
  return ListenWindowService;
}());

export {
  ListenWindowService,
};
